from __future__ import division, print_function, absolute_import

import os
import numpy as np
import logging
from dipy.io.image import load_nifti
from dipy.workflows.workflow import Workflow


class IoInfoFlow(Workflow):

    @classmethod
    def get_short_name(cls):
        return 'io_info'

    def run(self, input_files,
            b0_threshold=50, bvecs_tol=0.01, bshell_thr=100):

        """ Provides useful information about different files used in
        medical imaging. Any number of input files can be provided. The
        program identifies the type of file by its extension.

        Parameters
        ----------
        input_files : variable string
            Any number of Nifti1, bvals or bvecs files.
        b0_threshold : float, optional
            (default 50)
        bvecs_tol : float, optional
            Threshold used to check that norm(bvec) = 1 +/- bvecs_tol
            b-vectors are unit vectors (default 0.01)
        bshell_thr : float, optional
            Threshold for distinguishing b-values in different shells
            (default 100)
        """

        np.set_printoptions(3, suppress=True)

        for input_path in input_files:
            logging.info('------------------------------------------')
            logging.info('Looking at {0}'.format(input_path))
            logging.info('------------------------------------------')

            ipath_lower = input_path.lower()

            if ipath_lower.endswith('.nii') or ipath_lower.endswith('.nii.gz'):

                data, affine, img, vox_sz, affcodes = load_nifti(
                    input_path,
                    return_img=True,
                    return_voxsize=True,
                    return_coords=True)
                logging.info('Data size {0}'.format(data.shape))
                logging.info('Data type {0}'.format(data.dtype))
                logging.info('Data min {0} max {1} avg {2}'
                             .format(data.min(), data.max(), data.mean()))
                logging.info('2nd percentile {0} 98th percentile {1}'
                             .format(np.percentile(data, 2),
                                     np.percentile(data, 98)))
                logging.info('Native coordinate system {0}'
                             .format(''.join(affcodes)))
                logging.info('Affine to RAS1mm \n{0}'.format(affine))
                logging.info('Voxel size {0}'.format(np.array(vox_sz)))
                if np.sum(np.abs(np.diff(vox_sz))) > 0.1:
                    msg = \
                        'Voxel size is not isotropic. Please reslice.\n'
                    logging.warning(msg)

            if os.path.basename(input_path).lower().find('bval') > -1:
                bvals = np.loadtxt(input_path)
                logging.info('Bvalues \n{0}'.format(bvals))
                logging.info('Total number of bvalues {}'.format(len(bvals)))
                shells = np.sum(np.diff(np.sort(bvals)) > bshell_thr)
                logging.info('Number of gradient shells {0}'.format(shells))
                logging.info('Number of b0s {0} (b0_thr {1})\n'
                             .format(np.sum(bvals <= b0_threshold),
                                     b0_threshold))

            if os.path.basename(input_path).lower().find('bvec') > -1:

                bvecs = np.loadtxt(input_path)
                logging.info('Bvectors shape on disk is {0}'
                             .format(bvecs.shape))
                rows, cols = bvecs.shape
                if rows < cols:
                    bvecs = bvecs.T
                logging.info('Bvectors are \n{0}'.format(bvecs))
                norms = np.array([np.linalg.norm(bvec) for bvec in bvecs])
                res = np.where(
                        (norms <= 1 + bvecs_tol) & (norms >= 1 - bvecs_tol))
                ncl1 = np.sum(norms < 1 - bvecs_tol)
                logging.info('Total number of unit bvectors {0}'
                             .format(len(res[0])))
                logging.info('Total number of non-unit bvectors {0}\n'
                             .format(ncl1))

        np.set_printoptions()



from .direction_getter import DirectionGetter
from .localtracking import LocalTracking, ParticleFilteringTracking
from .tissue_classifier import (ActTissueClassifier,
                                BinaryTissueClassifier,
                                CmcTissueClassifier,
                                ConstrainedTissueClassifier,
                                ThresholdTissueClassifier,
                                TissueClassifier)

from dipy.tracking import utils

__all__ = ["ActTissueClassifier", "BinaryTissueClassifier",
           "CmcTissueClassifier", "ConstrainedTissueClassifier",
           "DirectionGetter", "LocalTracking", "ParticleFilteringTracking",
           "ThresholdTissueClassifier", "TissueClassifier"]



import nibabel as nib
from nibabel.streamlines import Field
from nibabel.orientations import aff2axcodes


def save_trk(fname, streamlines, affine, vox_size=None, shape=None, header=None):
    """ Saves tractogram files (*.trk)

    Parameters
    ----------
    fname : str
        output trk filename
    streamlines : list of 2D arrays, generator or ArraySequence
        Each 2D array represents a sequence of 3D points (points, 3).
    affine : array_like (4, 4)
        The mapping from voxel coordinates to streamline points.
    vox_size : array_like (3,), optional
        The sizes of the voxels in the reference image (default: None)
    shape : array, shape (dim,), optional
        The shape of the reference image (default: None)
    header : dict, optional
        Metadata associated to the tractogram file(*.trk). (default: None)
    """
    if vox_size is not None and shape is not None:
        if not isinstance(header, dict):
            header = {}
        header[Field.VOXEL_TO_RASMM] = affine.copy()
        header[Field.VOXEL_SIZES] = vox_size
        header[Field.DIMENSIONS] = shape
        header[Field.VOXEL_ORDER] = "".join(aff2axcodes(affine))

    tractogram = nib.streamlines.Tractogram(streamlines)
    tractogram.affine_to_rasmm = affine
    trk_file = nib.streamlines.TrkFile(tractogram, header=header)
    nib.streamlines.save(trk_file, fname)


def load_trk(filename):
    """ Loads tractogram files(*.trk)

    Parameters
    ----------
    filename : str
        input trk filename

    Returns
    -------
    streamlines : list of 2D arrays
        Each 2D array represents a sequence of 3D points (points, 3).
    hdr : dict
        header from a trk file
    """
    trk_file = nib.streamlines.load(filename)
    return trk_file.streamlines, trk_file.header



''' Utilities for testing '''
from os.path import dirname, abspath, join as pjoin
from dipy.testing.spherepoints import sphere_points
from dipy.testing.decorators import doctest_skip_parser
from numpy.testing import assert_array_equal
import numpy as np
from distutils.version import LooseVersion

# set path to example data
IO_DATA_PATH = abspath(pjoin(dirname(__file__),
                             '..', 'io', 'tests', 'data'))

# Allow failed import of nose if not now running tests
try:
    import nose.tools as nt
except ImportError:
    pass
else:
    from nose.tools import (assert_equal, assert_not_equal,
                            assert_true, assert_false, assert_raises)


def assert_arrays_equal(arrays1, arrays2):
    for arr1, arr2 in zip(arrays1, arrays2):
        assert_array_equal(arr1, arr2)

def setup_test():
    """ Set numpy print options to "legacy" for new versions of numpy

    If imported into a file, nosetest will run this before any doctests.

    References
    -----------
    https://github.com/numpy/numpy/commit/710e0327687b9f7653e5ac02d222ba62c657a718
    https://github.com/numpy/numpy/commit/734b907fc2f7af6e40ec989ca49ee6d87e21c495
    https://github.com/nipy/nibabel/pull/556
    """
    if LooseVersion(np.__version__) >= LooseVersion('1.14'):
        np.set_printoptions(legacy='1.13')



"""
==============================================================
Crossing invariant fiber response function with FORECAST model
==============================================================

We show how to obtain a voxel specific response function in the form of
axially symmetric tensor and the fODF using the FORECAST model from
[Anderson2005]_ , [Kaden2016]_ and [Zucchelli2017]_.

First import the necessary modules:
"""

import matplotlib.pyplot as plt
from dipy.reconst.forecast import ForecastModel
from dipy.viz import actor, window
from dipy.data import fetch_cenir_multib, read_cenir_multib, get_sphere

"""
Download and read the data for this tutorial.
Our implementation of FORECAST requires multi-shell data.
fetch_cenir_multib() provides data acquired using the shells at b-values 1000,
2000, and 3000 (see MAPMRI example for more information on this dataset).
"""

fetch_cenir_multib(with_raw=False)

bvals = [1000, 2000, 3000]
img, gtab = read_cenir_multib(bvals)
data = img.get_data()

"""
Let us consider only a single slice for the FORECAST fitting
"""

data_small = data[18:87, 51:52, 10:70]
mask_small = data_small[..., 0] > 1000

"""
Instantiate the FORECAST Model.

"sh_order" is the spherical harmonics order used for the fODF.

dec_alg is the spherical deconvolution algorithm used for the FORECAST basis fitting, in this case
we used the Constrained Spherical Deconvolution (CSD) algorithm.
"""

fm = ForecastModel(gtab, sh_order=6, dec_alg='CSD')

"""
Fit the FORECAST to the data
"""

f_fit = fm.fit(data_small, mask_small)

"""
Calculate the crossing invariant tensor indices [Kaden2016]_ : the parallel diffusivity,
the perpendicular diffusivity, the fractional anisotropy and the mean
diffusivity.
"""

d_par = f_fit.dpar
d_perp = f_fit.dperp
fa = f_fit.fractional_anisotropy()
md = f_fit.mean_diffusivity()

"""
Show the indices and save them in FORECAST_indices.png.
"""

fig = plt.figure(figsize=(6, 6))
ax1 = fig.add_subplot(2, 2, 1, title='parallel diffusivity')
ax1.set_axis_off()
ind = ax1.imshow(d_par[:, 0, :].T, interpolation='nearest',
                 origin='lower', cmap=plt.cm.gray)
plt.colorbar(ind, shrink=0.6)
ax2 = fig.add_subplot(2, 2, 2, title='perpendicular diffusivity')
ax2.set_axis_off()
ind = ax2.imshow(d_perp[:, 0, :].T, interpolation='nearest',
                 origin='lower', cmap=plt.cm.gray)
plt.colorbar(ind, shrink=0.6)
ax3 = fig.add_subplot(2, 2, 3, title='fractional anisotropy')
ax3.set_axis_off()
ind = ax3.imshow(fa[:, 0, :].T, interpolation='nearest',
                 origin='lower', cmap=plt.cm.gray)
plt.colorbar(ind, shrink=0.6)
ax4 = fig.add_subplot(2, 2, 4, title='mean diffusivity')
ax4.set_axis_off()
ind = ax4.imshow(md[:, 0, :].T, interpolation='nearest',
                 origin='lower', cmap=plt.cm.gray)
plt.colorbar(ind, shrink=0.6)
plt.savefig('FORECAST_indices.png', dpi=300, bbox_inches='tight')

"""
.. figure:: FORECAST_indices.png
   :align: center

   **FORECAST scalar indices**.

"""

"""
Load an ODF reconstruction sphere
"""

sphere = get_sphere('symmetric724')

"""
Compute the fODFs.
"""

odf = f_fit.odf(sphere)
print('fODF.shape (%d, %d, %d, %d)' % odf.shape)

"""
Display a part of the fODFs
"""

odf_actor = actor.odf_slicer(odf[16:36, :, 30:45], sphere=sphere,
                             colormap='plasma', scale=0.6)
odf_actor.display(y=0)
odf_actor.RotateX(-90)
ren = window.Renderer()
ren.add(odf_actor)
window.record(ren, out_path='fODFs.png', size=(600, 600), magnification=4)

"""
.. figure:: fODFs.png
   :align: center

   **Fiber Orientation Distribution Functions, in a small ROI of the brain**.

References
----------

.. [Anderson2005] Anderson A. W., "Measurement of Fiber Orientation Distributions
       Using High Angular Resolution Diffusion Imaging", Magnetic
       Resonance in Medicine, 2005.

.. [Kaden2016] Kaden E. et al., "Quantitative Mapping of the Per-Axon Diffusion
       Coefficients in Brain White Matter", Magnetic Resonance in
       Medicine, 2016.

.. [Zucchelli2017] Zucchelli E. et al., "A generalized SMT-based framework for
       Diffusion MRI microstructural model estimation", MICCAI Workshop
       on Computational DIFFUSION MRI (CDMRI), 2017.

.. include:: ../links_names.inc

"""



import numpy as np
from nose.tools import assert_false, assert_equal
from os.path import join

import nibabel as nib
from nibabel.tmpdirs import TemporaryDirectory

from dipy.data import get_data
from dipy.io.image import save_nifti
from dipy.workflows.mask import MaskFlow
from dipy.workflows.reconst import ReconstCSDFlow
from dipy.workflows.tracking import DetTrackPAMFlow


def test_det_track():
    with TemporaryDirectory() as out_dir:
        data_path, bval_path, bvec_path = get_data('small_64D')
        vol_img = nib.load(data_path)
        volume = vol_img.get_data()
        mask = np.ones_like(volume[:, :, :, 0])
        mask_img = nib.Nifti1Image(mask.astype(np.uint8), vol_img.affine)
        mask_path = join(out_dir, 'tmp_mask.nii.gz')
        nib.save(mask_img, mask_path)

        reconst_csd_flow = ReconstCSDFlow()
        reconst_csd_flow.run(data_path, bval_path, bvec_path, mask_path,
                             out_dir=out_dir, extract_pam_values=True)

        pam_path = reconst_csd_flow.last_generated_outputs['out_pam']
        gfa_path = reconst_csd_flow.last_generated_outputs['out_gfa']

        # Create seeding mask by thresholding the gfa
        mask_flow = MaskFlow()
        mask_flow.run(gfa_path, 0.8, out_dir=out_dir)
        seeds_path = mask_flow.last_generated_outputs['out_mask']

        # Put identity in gfa path to prevent impossible to use
        # local tracking because of affine containing shearing.
        gfa_img = nib.load(gfa_path)
        save_nifti(gfa_path, gfa_img.get_data(), np.eye(4), gfa_img.header)

        # Test tracking with pam no sh
        det_track_pam = DetTrackPAMFlow()
        assert_equal(det_track_pam.get_short_name(), 'det_track')
        det_track_pam.run(pam_path, gfa_path, seeds_path)
        tractogram_path = \
            det_track_pam.last_generated_outputs['out_tractogram']
        assert_false(is_tractogram_empty(tractogram_path))

        # Test tracking with pam with sh
        det_track_pam.run(pam_path, gfa_path, seeds_path, use_sh=True)
        tractogram_path = \
            det_track_pam.last_generated_outputs['out_tractogram']
        assert_false(is_tractogram_empty(tractogram_path))


def is_tractogram_empty(tractogram_path):
    tractogram_file = \
        nib.streamlines.load(tractogram_path)

    return len(tractogram_file.tractogram) == 0


if __name__ == '__main__':
    test_det_track()



from __future__ import division
from _warnings import warn

import os
import glob
import numpy as np

from dipy.data import read_viz_icons
from dipy.viz.interactor import CustomInteractorStyle

from dipy.utils.optpkg import optional_package

# Allow import, but disable doctests if we don't have vtk.
vtk, have_vtk, setup_module = optional_package('vtk')

if have_vtk:
    version = vtk.vtkVersion.GetVTKVersion()
    major_version = vtk.vtkVersion.GetVTKMajorVersion()

TWO_PI = 2 * np.pi


class UI(object):
    """ An umbrella class for all UI elements.

    While adding UI elements to the renderer, we go over all the sub-elements
    that come with it and add those to the renderer automatically.

    Attributes
    ----------
    ui_param : object
        This is an attribute that can be passed to the UI object by the
        interactor.
    ui_list : list of :class:`UI`
        This is used when there are more than one UI elements inside
        a UI element. They're all automatically added to the renderer at the
        same time as this one.
    parent_ui: UI
        Reference to the parent UI element. This is useful of there is a parent
        UI element and its reference needs to be passed down to the child.
    on_left_mouse_button_pressed: function
        Callback function for when the left mouse button is pressed.
    on_left_mouse_button_released: function
        Callback function for when the left mouse button is released.
    on_left_mouse_button_clicked: function
        Callback function for when clicked using the left mouse button
        (i.e. pressed -> released).
    on_left_mouse_button_dragged: function
        Callback function for when dragging using the left mouse button.
    on_right_mouse_button_pressed: function
        Callback function for when the right mouse button is pressed.
    on_right_mouse_button_released: function
        Callback function for when the right mouse button is released.
    on_right_mouse_button_clicked: function
        Callback function for when clicking using the right mouse button
        (i.e. pressed -> released).
    on_right_mouse_button_dragged: function
        Callback function for when dragging using the right mouse button.

    """

    def __init__(self):
        self.ui_param = None
        self.ui_list = list()

        self.parent_ui = None
        self._callbacks = []

        self.left_button_state = "released"
        self.right_button_state = "released"

        self.on_left_mouse_button_pressed = lambda i_ren, obj, element: None
        self.on_left_mouse_button_dragged = lambda i_ren, obj, element: None
        self.on_left_mouse_button_released = lambda i_ren, obj, element: None
        self.on_left_mouse_button_clicked = lambda i_ren, obj, element: None
        self.on_right_mouse_button_pressed = lambda i_ren, obj, element: None
        self.on_right_mouse_button_released = lambda i_ren, obj, element: None
        self.on_right_mouse_button_clicked = lambda i_ren, obj, element: None
        self.on_right_mouse_button_dragged = lambda i_ren, obj, element: None
        self.on_key_press = lambda i_ren, obj, element: None

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        msg = "Subclasses of UI must implement `get_actors(self)`."
        raise NotImplementedError(msg)

    def add_to_renderer(self, ren):
        """ Allows UI objects to add their own props to the renderer.

        Parameters
        ----------
        ren : renderer

        """
        ren.add(*self.get_actors())

        # Get a hold on the current interactor style.
        iren = ren.GetRenderWindow().GetInteractor().GetInteractorStyle()

        for callback in self._callbacks:
            if not isinstance(iren, CustomInteractorStyle):
                msg = ("The ShowManager requires `CustomInteractorStyle` in"
                       " order to use callbacks.")
                raise TypeError(msg)

            iren.add_callback(*callback, args=[self])

    def add_callback(self, prop, event_type, callback, priority=0):
        """ Adds a callback to a specific event for this UI component.

        Parameters
        ----------
        prop : vtkProp
            The prop on which is callback is to be added.
        event_type : string
            The event code.
        callback : function
            The callback function.
        priority : int
            Higher number is higher priority.

        """
        # Actually since we need an interactor style we will add the callback
        # only when this UI component is added to the renderer.
        self._callbacks.append((prop, event_type, callback, priority))

    def set_center(self, position):
        """ Sets the center of the UI component

        Parameters
        ----------
        position : (float, float)
            These are the x and y coordinates respectively, with the
            origin at the bottom left.

        """
        msg = "Subclasses of UI must implement `set_center(self, position)`."
        raise NotImplementedError(msg)

    def set_visibility(self, visibility):
        """ Sets visibility of this UI component and all its sub-components.

        """
        for actor in self.get_actors():
            actor.SetVisibility(visibility)

    def handle_events(self, actor):
        self.add_callback(actor, "LeftButtonPressEvent", self.left_button_click_callback)
        self.add_callback(actor, "LeftButtonReleaseEvent", self.left_button_release_callback)
        self.add_callback(actor, "RightButtonPressEvent", self.right_button_click_callback)
        self.add_callback(actor, "RightButtonReleaseEvent", self.right_button_release_callback)
        self.add_callback(actor, "MouseMoveEvent", self.mouse_move_callback)
        self.add_callback(actor, "KeyPressEvent", self.key_press_callback)

    @staticmethod
    def left_button_click_callback(i_ren, obj, self):
        self.left_button_state = "pressing"
        self.on_left_mouse_button_pressed(i_ren, obj, self)
        i_ren.event.abort()

    @staticmethod
    def left_button_release_callback(i_ren, obj, self):
        if self.left_button_state == "pressing":
            self.on_left_mouse_button_clicked(i_ren, obj, self)
        self.left_button_state = "released"
        self.on_left_mouse_button_released(i_ren, obj, self)

    @staticmethod
    def right_button_click_callback(i_ren, obj, self):
        self.right_button_state = "pressing"
        self.on_right_mouse_button_pressed(i_ren, obj, self)
        i_ren.event.abort()

    @staticmethod
    def right_button_release_callback(i_ren, obj, self):
        if self.right_button_state == "pressing":
            self.on_right_mouse_button_clicked(i_ren, obj, self)
        self.right_button_state = "released"
        self.on_right_mouse_button_released(i_ren, obj, self)

    @staticmethod
    def mouse_move_callback(i_ren, obj, self):
        if self.left_button_state == "pressing" or self.left_button_state == "dragging":
            self.left_button_state = "dragging"
            self.on_left_mouse_button_dragged(i_ren, obj, self)
        elif self.right_button_state == "pressing" or self.right_button_state == "dragging":
            self.right_button_state = "dragging"
            self.on_right_mouse_button_dragged(i_ren, obj, self)
        else:
            pass

    @staticmethod
    def key_press_callback(i_ren, obj, self):
        self.on_key_press(i_ren, obj, self)


class Button2D(UI):
    """ A 2D overlay button and is of type vtkTexturedActor2D.
    Currently supports:
    - Multiple icons.
    - Switching between icons.

    Attributes
    ----------
    size: (float, float)
        Button size (width, height) in pixels.

    """

    def __init__(self, icon_fnames, size=(30, 30)):
        """
        Parameters
        ----------
        size : 2-tuple of int, optional
            Button size.
        icon_fnames : dict
            {iconname : filename, iconname : filename, ...}

        """
        super(Button2D, self).__init__()
        self.icon_extents = dict()
        self.icons = self.__build_icons(icon_fnames)
        self.icon_names = list(self.icons.keys())
        self.current_icon_id = 0
        self.current_icon_name = self.icon_names[self.current_icon_id]
        self.actor = self.build_actor(self.icons[self.current_icon_name])
        self.size = size
        self.handle_events(self.actor)

    def __build_icons(self, icon_fnames):
        """ Converts file names to vtkImageDataGeometryFilters.

        A pre-processing step to prevent re-read of file names during every
        state change.

        Parameters
        ----------
        icon_fnames : dict
            {iconname: filename, iconname: filename, ...}

        Returns
        -------
        icons : dict
            A dictionary of corresponding vtkImageDataGeometryFilters.

        """
        icons = {}
        for icon_name, icon_fname in icon_fnames.items():
            if icon_fname.split(".")[-1] not in ["png", "PNG"]:
                error_msg = "A specified icon file is not in the PNG format. SKIPPING."
                warn(Warning(error_msg))
            else:
                png = vtk.vtkPNGReader()
                png.SetFileName(icon_fname)
                png.Update()
                icons[icon_name] = png.GetOutput()

        return icons

    @property
    def size(self):
        """ Gets the button size.

        """
        return self._size

    @size.setter
    def size(self, size):
        """ Sets the button size.

        Parameters
        ----------
        size : (float, float)
            Button size (width, height) in pixels.

        """
        self._size = np.asarray(size)

        # Update actor.
        self.texture_points.SetPoint(0, 0, 0, 0.0)
        self.texture_points.SetPoint(1, size[0], 0, 0.0)
        self.texture_points.SetPoint(2, size[0], size[1], 0.0)
        self.texture_points.SetPoint(3, 0, size[1], 0.0)
        self.texture_polydata.SetPoints(self.texture_points)

    @property
    def color(self):
        """ Gets the button's color.

        """
        color = self.actor.GetProperty().GetColor()
        return np.asarray(color)

    @color.setter
    def color(self, color):
        """ Sets the button's color.

        Parameters
        ----------
        color : (float, float, float)
            RGB. Must take values in [0, 1].

        """
        self.actor.GetProperty().SetColor(*color)

    def scale(self, size):
        """ Scales the button.

        Parameters
        ----------
        size : (float, float)
            Scaling factor (width, height) in pixels.

        """
        self.size *= size

    def build_actor(self, icon):
        """ Return an image as a 2D actor with a specific position.

        Parameters
        ----------
        icon : :class:`vtkImageData`

        Returns
        -------
        :class:`vtkTexturedActor2D`

        """
        # This is highly inspired by
        # https://github.com/Kitware/VTK/blob/c3ec2495b183e3327820e927af7f8f90d34c3474\
        # /Interaction/Widgets/vtkBalloonRepresentation.cxx#L47

        self.texture_polydata = vtk.vtkPolyData()
        self.texture_points = vtk.vtkPoints()
        self.texture_points.SetNumberOfPoints(4)
        self.size = icon.GetExtent()

        polys = vtk.vtkCellArray()
        polys.InsertNextCell(4)
        polys.InsertCellPoint(0)
        polys.InsertCellPoint(1)
        polys.InsertCellPoint(2)
        polys.InsertCellPoint(3)
        self.texture_polydata.SetPolys(polys)

        tc = vtk.vtkFloatArray()
        tc.SetNumberOfComponents(2)
        tc.SetNumberOfTuples(4)
        tc.InsertComponent(0, 0, 0.0)
        tc.InsertComponent(0, 1, 0.0)
        tc.InsertComponent(1, 0, 1.0)
        tc.InsertComponent(1, 1, 0.0)
        tc.InsertComponent(2, 0, 1.0)
        tc.InsertComponent(2, 1, 1.0)
        tc.InsertComponent(3, 0, 0.0)
        tc.InsertComponent(3, 1, 1.0)
        self.texture_polydata.GetPointData().SetTCoords(tc)

        texture_mapper = vtk.vtkPolyDataMapper2D()
        if major_version <= 5:
            texture_mapper.SetInput(self.texture_polydata)
        else:
            texture_mapper.SetInputData(self.texture_polydata)

        button = vtk.vtkTexturedActor2D()
        button.SetMapper(texture_mapper)

        self.texture = vtk.vtkTexture()
        button.SetTexture(self.texture)

        button_property = vtk.vtkProperty2D()
        button_property.SetOpacity(1.0)
        button.SetProperty(button_property)

        self.set_icon(icon)
        return button

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        return [self.actor]

    def set_icon(self, icon):
        """ Modifies the icon used by the vtkTexturedActor2D.

        Parameters
        ----------
        icon : imageDataGeometryFilter

        """
        if major_version <= 5:
            self.texture.SetInput(icon)
        else:
            self.texture.SetInputData(icon)

    def next_icon_name(self):
        """ Returns the next icon name while cycling through icons.

        """
        self.current_icon_id += 1
        if self.current_icon_id == len(self.icons):
            self.current_icon_id = 0
        self.current_icon_name = self.icon_names[self.current_icon_id]

    def next_icon(self):
        """ Increments the state of the Button.

            Also changes the icon.

        """
        self.next_icon_name()
        self.set_icon(self.icons[self.current_icon_name])

    def set_center(self, position):
        """ Sets the icon center to position.

        Parameters
        ----------
        position : (float, float)
            The new center of the button (x, y).

        """
        new_position = np.asarray(position) - self.size / 2.
        self.actor.SetPosition(*new_position)


class Rectangle2D(UI):
    """ A 2D rectangle sub-classed from UI.
    Uses vtkPolygon.

    Attributes
    ----------
    size : (float, float)
        The size of the rectangle (height, width) in pixels.

    """

    def __init__(self, size, center=(0, 0), color=(1, 1, 1), opacity=1.0):
        """ Initializes a rectangle.

        Parameters
        ----------
        size : (float, float)
            The size of the rectangle (height, width) in pixels.
        center : (float, float)
            The center of the rectangle (x, y).
        color : (float, float, float)
            Must take values in [0, 1].
        opacity : float
            Must take values in [0, 1].

        """
        super(Rectangle2D, self).__init__()
        self.size = size
        self.actor = self.build_actor(size=size)
        self.color = color
        self.set_center(center)
        self.opacity = opacity
        self.handle_events(self.actor)

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        return [self.actor]

    def build_actor(self, size):
        """ Builds the text actor.

        Parameters
        ----------
        size : (float, float)
            The size of the rectangle (height, width) in pixels.

        Returns
        -------
        :class:`vtkActor2D`

        """
        # Setup four points
        points = vtk.vtkPoints()
        points.InsertNextPoint(0, 0, 0)
        points.InsertNextPoint(size[0], 0, 0)
        points.InsertNextPoint(size[0], size[1], 0)
        points.InsertNextPoint(0, size[1], 0)

        # Create the polygon
        polygon = vtk.vtkPolygon()
        polygon.GetPointIds().SetNumberOfIds(4)  # make a quad
        polygon.GetPointIds().SetId(0, 0)
        polygon.GetPointIds().SetId(1, 1)
        polygon.GetPointIds().SetId(2, 2)
        polygon.GetPointIds().SetId(3, 3)

        # Add the polygon to a list of polygons
        polygons = vtk.vtkCellArray()
        polygons.InsertNextCell(polygon)

        # Create a PolyData
        polygonPolyData = vtk.vtkPolyData()
        polygonPolyData.SetPoints(points)
        polygonPolyData.SetPolys(polygons)

        # Create a mapper and actor
        mapper = vtk.vtkPolyDataMapper2D()
        if vtk.VTK_MAJOR_VERSION <= 5:
            mapper.SetInput(polygonPolyData)
        else:
            mapper.SetInputData(polygonPolyData)

        actor = vtk.vtkActor2D()
        actor.SetMapper(mapper)

        return actor

    def set_position(self, position):
        self.actor.SetPosition(*position)

    def set_center(self, position):
        """ Sets the center to position.

        Parameters
        ----------
        position : (float, float)
            The new center of the rectangle (x, y).

        """
        self.actor.SetPosition(position[0] - self.size[0] / 2,
                               position[1] - self.size[1] / 2)

    @property
    def color(self):
        """ Gets the rectangle's color.

        """
        color = self.actor.GetProperty().GetColor()
        return np.asarray(color)

    @color.setter
    def color(self, color):
        """ Sets the rectangle's color.

        Parameters
        ----------
        color : (float, float, float)
            RGB. Must take values in [0, 1].

        """
        self.actor.GetProperty().SetColor(*color)

    @property
    def opacity(self):
        """ Gets the rectangle's opacity.

        """
        return self.actor.GetProperty().GetOpacity()

    @opacity.setter
    def opacity(self, opacity):
        """ Sets the rectangle's opacity.

        Parameters
        ----------
        opacity : float
            Degree of transparency. Must be between [0, 1].

        """
        self.actor.GetProperty().SetOpacity(opacity)

    @property
    def position(self):
        """ Gets text actor position.

        Returns
        -------
        (float, float)
            The current actor position. (x, y) in pixels.

        """
        return self.actor.GetPosition()

    @position.setter
    def position(self, position):
        """ Set text actor position.

        Parameters
        ----------
        position : (float, float)
            The new position. (x, y) in pixels.

        """
        self.actor.SetPosition(*position)


class Panel2D(UI):
    """ A 2D UI Panel.

    Can contain one or more UI elements.

    Attributes
    ----------
    center : (float, float)
        The center of the panel (x, y).
    size : (float, float)
        The size of the panel (width, height) in pixels.
    alignment : [left, right]
        Alignment of the panel with respect to the overall screen.

    """

    def __init__(self, center, size, color=(0.1, 0.1, 0.1), opacity=0.7, align="left"):
        """
        Parameters
        ----------
        center : (float, float)
            The center of the panel (x, y).
        size : (float, float)
            The size of the panel (width, height) in pixels.
        color : (float, float, float)
            Must take values in [0, 1].
        opacity : float
            Must take values in [0, 1].
        align : [left, right]
            Alignment of the panel with respect to the overall screen.

        """
        super(Panel2D, self).__init__()
        self.center = center
        self.size = size
        self.lower_limits = (self.center[0] - self.size[0] / 2,
                             self.center[1] - self.size[1] / 2)

        self.panel = Rectangle2D(size=size, center=center, color=color,
                                 opacity=opacity)

        self.element_positions = []
        self.element_positions.append([self.panel, 'relative', 0.5, 0.5])
        self.alignment = align

        self.handle_events(self.panel.actor)

        self.on_left_mouse_button_pressed = self.left_button_pressed
        self.on_left_mouse_button_dragged = self.left_button_dragged

    def add_to_renderer(self, ren):
        """ Allows UI objects to add their own props to the renderer.

        Here, we add only call add_to_renderer for the additional components.

        Parameters
        ----------
        ren : renderer

        """
        super(Panel2D, self).add_to_renderer(ren)
        for ui_item in self.ui_list:
            ui_item.add_to_renderer(ren)

    def get_actors(self):
        """ Returns the panel actor.

        """
        return [self.panel.actor]

    def add_element(self, element, position_type, position):
        """ Adds an element to the panel.

        The center of the rectangular panel is its bottom lower position.

        Parameters
        ----------
        element : UI
            The UI item to be added.
        position_type: string
            'absolute' or 'relative'
        position : (float, float)
            Absolute for absolute and relative for relative

        """
        self.ui_list.append(element)
        if position_type == 'relative':
            self.element_positions.append([element, position_type, position[0], position[1]])
            element.set_center((self.lower_limits[0] + position[0] * self.size[0],
                                self.lower_limits[1] + position[1] * self.size[1]))
        elif position_type == 'absolute':
            self.element_positions.append([element, position_type, position[0], position[1]])
            element.set_center((position[0], position[1]))
        else:
            raise ValueError("Position can only be absolute or relative")

    def set_center(self, position):
        """ Sets the panel center to position.

        The center of the rectangular panel is its bottom lower position.

        Parameters
        ----------
        position : (float, float)
            The new center of the panel (x, y).

        """
        shift = [position[0] - self.center[0], position[1] - self.center[1]]
        self.center = position
        self.lower_limits = (position[0] - self.size[0] / 2, position[1] - self.size[1] / 2)
        for ui_element in self.element_positions:
            if ui_element[1] == 'relative':
                ui_element[0].set_center((self.lower_limits[0] + ui_element[2] * self.size[0],
                                          self.lower_limits[1] + ui_element[3] * self.size[1]))
            elif ui_element[1] == 'absolute':
                ui_element[2] += shift[0]
                ui_element[3] += shift[1]
                ui_element[0].set_center((ui_element[2], ui_element[3]))

    @staticmethod
    def left_button_pressed(i_ren, obj, panel2d_object):
        click_position = i_ren.event.position
        panel2d_object.ui_param = (click_position[0] -
                                   panel2d_object.panel.actor.GetPosition()[0] -
                                   panel2d_object.panel.size[0] / 2,
                                   click_position[1] -
                                   panel2d_object.panel.actor.GetPosition()[1] -
                                   panel2d_object.panel.size[1] / 2)
        i_ren.event.abort()  # Stop propagating the event.

    @staticmethod
    def left_button_dragged(i_ren, obj, panel2d_object):
        click_position = i_ren.event.position
        if panel2d_object.ui_param is not None:
            panel2d_object.set_center((click_position[0] - panel2d_object.ui_param[0],
                                       click_position[1] - panel2d_object.ui_param[1]))
        i_ren.force_render()

    def re_align(self, window_size_change):
        """ Re-organises the elements in case the window size is changed.

        Parameters
        ----------
        window_size_change : (int, int)
            New window size (width, height) in pixels.

        """
        if self.alignment == "left":
            pass
        elif self.alignment == "right":
            self.set_center((self.center[0] + window_size_change[0],
                             self.center[1] + window_size_change[1]))
        else:
            raise ValueError("You can only left-align or right-align objects in a panel.")


class TextBlock2D(UI):
    """ Wraps over the default vtkTextActor and helps setting the text.

    Contains member functions for text formatting.

    Attributes
    ----------
    actor : :class:`vtkTextActor`
        The text actor.
    message : str
        The initial text while building the actor.
    position : (float, float)
        (x, y) in pixels.
    color : (float, float, float)
        RGB: Values must be between 0-1.
    bg_color : (float, float, float)
        RGB: Values must be between 0-1.
    font_size : int
        Size of the text font.
    font_family : str
        Currently only supports Arial.
    justification : str
        left, right or center.
    vertical_justification : str
        bottom, middle or top.
    bold : bool
        Makes text bold.
    italic : bool
        Makes text italicised.
    shadow : bool
        Adds text shadow.
    """

    def __init__(self, text="Text Block", font_size=18, font_family='Arial',
                 justification='left', vertical_justification="bottom",
                 bold=False, italic=False, shadow=False,
                 color=(1, 1, 1), bg_color=None, position=(0, 0)):
        """
        Parameters
        ----------
        text : str
            The initial text while building the actor.
        position : (float, float)
            (x, y) in pixels.
        color : (float, float, float)
            RGB: Values must be between 0-1.
        bg_color : (float, float, float)
            RGB: Values must be between 0-1.
        font_size : int
            Size of the text font.
        font_family : str
            Currently only supports Arial.
        justification : str
            left, right or center.
        vertical_justification : str
            bottom, middle or top.
        bold : bool
            Makes text bold.
        italic : bool
            Makes text italicised.
        shadow : bool
            Adds text shadow.
        """
        super(TextBlock2D, self).__init__()
        self.actor = vtk.vtkTextActor()

        self._background = None  # For VTK < 7
        self.position = position
        self.color = color
        self.background_color = bg_color
        self.font_size = font_size
        self.font_family = font_family
        self.justification = justification
        self.bold = bold
        self.italic = italic
        self.shadow = shadow
        self.vertical_justification = vertical_justification
        self.message = text

    def get_actor(self):
        """ Returns the actor composing this element.

        Returns
        -------
        :class:`vtkTextActor`
            The actor composing this class.
        """
        return self.actor

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        if self._background is not None:
            return [self._background, self.actor]

        return [self.actor]

    @property
    def message(self):
        """ Gets message from the text.

        Returns
        -------
        str
            The current text message.

        """
        return self.actor.GetInput()

    @message.setter
    def message(self, text):
        """ Sets the text message.

        Parameters
        ----------
        text : str
            The message to be set.

        """
        self.actor.SetInput(text)

    @property
    def font_size(self):
        """ Gets text font size.

        Returns
        ----------
        int
            Text font size.

        """
        return self.actor.GetTextProperty().GetFontSize()

    @font_size.setter
    def font_size(self, size):
        """ Sets font size.

        Parameters
        ----------
        size : int
            Text font size.

        """
        self.actor.GetTextProperty().SetFontSize(size)

    @property
    def font_family(self):
        """ Gets font family.

        Returns
        ----------
        str
            Text font family.

        """
        return self.actor.GetTextProperty().GetFontFamilyAsString()

    @font_family.setter
    def font_family(self, family='Arial'):
        """ Sets font family.

        Currently Arial and Courier are supported.

        Parameters
        ----------
        family : str
            The font family.

        """
        if family == 'Arial':
            self.actor.GetTextProperty().SetFontFamilyToArial()
        elif family == 'Courier':
            self.actor.GetTextProperty().SetFontFamilyToCourier()
        else:
            raise ValueError("Font not supported yet: {}.".format(family))

    @property
    def justification(self):
        """ Gets text justification.

        Returns
        -------
        str
            Text justification.

        """
        justification = self.actor.GetTextProperty().GetJustificationAsString()
        if justification == 'Left':
            return "left"
        elif justification == 'Centered':
            return "center"
        elif justification == 'Right':
            return "right"

    @justification.setter
    def justification(self, justification):
        """ Justifies text.

        Parameters
        ----------
        justification : str
            Possible values are left, right, center.

        """
        text_property = self.actor.GetTextProperty()
        if justification == 'left':
            text_property.SetJustificationToLeft()
        elif justification == 'center':
            text_property.SetJustificationToCentered()
        elif justification == 'right':
            text_property.SetJustificationToRight()
        else:
            raise ValueError("Text can only be justified left, right and center.")

    @property
    def vertical_justification(self):
        """ Gets text vertical justification.

        Returns
        -------
        str
            Text vertical justification.

        """
        text_property = self.actor.GetTextProperty()
        vjustification = text_property.GetVerticalJustificationAsString()
        if vjustification == 'Bottom':
            return "bottom"
        elif vjustification == 'Centered':
            return "middle"
        elif vjustification == 'Top':
            return "top"

    @vertical_justification.setter
    def vertical_justification(self, vertical_justification):
        """ Justifies text vertically.

        Parameters
        ----------
        vertical_justification : str
            Possible values are bottom, middle, top.

        """
        text_property = self.actor.GetTextProperty()
        if vertical_justification == 'bottom':
            text_property.SetVerticalJustificationToBottom()
        elif vertical_justification == 'middle':
            text_property.SetVerticalJustificationToCentered()
        elif vertical_justification == 'top':
            text_property.SetVerticalJustificationToTop()
        else:
            msg = "Vertical justification must be: bottom, middle or top."
            raise ValueError(msg)

    @property
    def bold(self):
        """ Returns whether the text is bold.

        Returns
        -------
        bool
            Text is bold if True.

        """
        return self.actor.GetTextProperty().GetBold()

    @bold.setter
    def bold(self, flag):
        """ Bolds/un-bolds text.

        Parameters
        ----------
        flag : bool
            Sets text bold if True.

        """
        self.actor.GetTextProperty().SetBold(flag)

    @property
    def italic(self):
        """ Returns whether the text is italicised.

        Returns
        -------
        bool
            Text is italicised if True.

        """
        return self.actor.GetTextProperty().GetItalic()

    @italic.setter
    def italic(self, flag):
        """ Italicises/un-italicises text.

        Parameters
        ----------
        flag : bool
            Italicises text if True.

        """
        self.actor.GetTextProperty().SetItalic(flag)

    @property
    def shadow(self):
        """ Returns whether the text has shadow.

        Returns
        -------
        bool
            Text is shadowed if True.

        """
        return self.actor.GetTextProperty().GetShadow()

    @shadow.setter
    def shadow(self, flag):
        """ Adds/removes text shadow.

        Parameters
        ----------
        flag : bool
            Shadows text if True.

        """
        self.actor.GetTextProperty().SetShadow(flag)

    @property
    def color(self):
        """ Gets text color.

        Returns
        -------
        (float, float, float)
            Returns text color in RGB.

        """
        return self.actor.GetTextProperty().GetColor()

    @color.setter
    def color(self, color=(1, 0, 0)):
        """ Set text color.

        Parameters
        ----------
        color : (float, float, float)
            RGB: Values must be between 0-1.

        """
        self.actor.GetTextProperty().SetColor(*color)

    @property
    def background_color(self):
        """ Gets background color.

        Returns
        -------
        (float, float, float) or None
            If None, there no background color.
            Otherwise, background color in RGB.

        """
        if major_version < 7:
            if self._background is None:
                return None

            return self._background.GetProperty().GetColor()

        if self.actor.GetTextProperty().GetBackgroundOpacity() == 0:
            return None

        return self.actor.GetTextProperty().GetBackgroundColor()

    @background_color.setter
    def background_color(self, color):
        """ Set text color.

        Parameters
        ----------
        color : (float, float, float) or None
            If None, remove background.
            Otherwise, RGB values (must be between 0-1).

        """

        if color is None:
            # Remove background.
            if major_version < 7:
                self._background = None
            else:
                self.actor.GetTextProperty().SetBackgroundOpacity(0.)

        else:
            if major_version < 7:
                self._background = vtk.vtkActor2D()
                self._background.GetProperty().SetColor(*color)
                self._background.GetProperty().SetOpacity(1)
                self._background.SetMapper(self.actor.GetMapper())
                self._background.SetPosition(*self.actor.GetPosition())

            else:
                self.actor.GetTextProperty().SetBackgroundColor(*color)
                self.actor.GetTextProperty().SetBackgroundOpacity(1.)

    @property
    def position(self):
        """ Gets text actor position.

        Returns
        -------
        (float, float)
            The current actor position. (x, y) in pixels.

        """
        return self.actor.GetPosition()

    @position.setter
    def position(self, position):
        """ Set text actor position.

        Parameters
        ----------
        position : (float, float)
            The new position. (x, y) in pixels.

        """
        self.actor.SetPosition(*position)
        if self._background is not None:
            self._background.SetPosition(*self.actor.GetPosition())

    def set_center(self, position):
        """ Sets the text center to position.

        Parameters
        ----------
        position : (float, float)

        """
        self.position = position


class TextBox2D(UI):
    """ An editable 2D text box that behaves as a UI component.

    Currently supports:
    - Basic text editing.
    - Cursor movements.
    - Single and multi-line text boxes.
    - Pre text formatting (text needs to be formatted beforehand).

    Attributes
    ----------
    text : str
        The current text state.
    actor : :class:`vtkActor2d`
        The text actor.
    width : int
        The number of characters in a single line of text.
    height : int
        The number of lines in the textbox.
    window_left : int
        Left limit of visible text in the textbox.
    window_right : int
        Right limit of visible text in the textbox.
    caret_pos : int
        Position of the caret in the text.
    init : bool
        Flag which says whether the textbox has just been initialized.

    """
    def __init__(self, width, height, text="Enter Text", position=(100, 10),
                 color=(0, 0, 0), font_size=18, font_family='Arial',
                 justification='left', bold=False,
                 italic=False, shadow=False):
        """
        Parameters
        ----------
        width : int
            The number of characters in a single line of text.
        height : int
            The number of lines in the textbox.
        text : str
            The initial text while building the actor.
        position : (float, float)
            (x, y) in pixels.
        color : (float, float, float)
            RGB: Values must be between 0-1.
        font_size : int
            Size of the text font.
        font_family : str
            Currently only supports Arial.
        justification : str
            left, right or center.
        bold : bool
            Makes text bold.
        italic : bool
            Makes text italicised.
        shadow : bool
            Adds text shadow.

        """
        super(TextBox2D, self).__init__()
        self.text = text
        self.actor = self.build_actor(self.text, position, color, font_size,
                                      font_family, justification, bold, italic, shadow)
        self.width = width
        self.height = height
        self.window_left = 0
        self.window_right = 0
        self.caret_pos = 0
        self.init = True

        self.handle_events(self.actor.get_actor())

        self.on_left_mouse_button_pressed = self.left_button_press
        self.on_key_press = self.key_press

    def build_actor(self, text, position, color, font_size,
                    font_family, justification, bold, italic, shadow):

        """ Builds a text actor.

        Parameters
        ----------
        text : str
            The initial text while building the actor.
        position : (float, float)
            (x, y) in pixels.
        color : (float, float, float)
            RGB: Values must be between 0-1.
        font_size : int
            Size of the text font.
        font_family : str
            Currently only supports Arial.
        justification : str
            left, right or center.
        bold : bool
            Makes text bold.
        italic : bool
            Makes text italicised.
        shadow : bool
            Adds text shadow.

        Returns
        -------
        :class:`TextBlock2D`

        """
        text_block = TextBlock2D()
        text_block.position = position
        text_block.message = text
        text_block.font_size = font_size
        text_block.font_family = font_family
        text_block.justification = justification
        text_block.bold = bold
        text_block.italic = italic
        text_block.shadow = shadow

        if major_version >= 7:
            text_block.actor.GetTextProperty().SetBackgroundColor(1, 1, 1)
            text_block.actor.GetTextProperty().SetBackgroundOpacity(1.0)
            text_block.color = color

        return text_block

    def set_message(self, message):
        """ Set custom text to textbox.

        Parameters
        ----------
        message: str
            The custom message to be set.

        """
        self.text = message
        self.actor.message = message
        self.init = False
        self.window_right = len(self.text)
        self.window_left = 0
        self.caret_pos = self.window_right

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        return [self.actor.get_actor()]

    def width_set_text(self, text):
        """ Adds newlines to text where necessary.

        This is needed for multi-line text boxes.

        Parameters
        ----------
        text : str
            The final text to be formatted.

        Returns
        -------
        str
            A multi line formatted text.

        """
        multi_line_text = ""
        for i in range(len(text)):
            multi_line_text += text[i]
            if (i + 1) % self.width == 0:
                multi_line_text += "\n"
        return multi_line_text.rstrip("\n")

    def handle_character(self, character):
        """ Main driving function that handles button events.

        # TODO: Need to handle all kinds of characters like !, +, etc.

        Parameters
        ----------
        character : str

        """
        if character.lower() == "return":
            self.render_text(False)
            return True
        if character.lower() == "backspace":
            self.remove_character()
        elif character.lower() == "left":
            self.move_left()
        elif character.lower() == "right":
            self.move_right()
        else:
            self.add_character(character)
        self.render_text()
        return False

    def move_caret_right(self):
        """ Moves the caret towards right.

        """
        self.caret_pos = min(self.caret_pos + 1, len(self.text))

    def move_caret_left(self):
        """ Moves the caret towards left.

        """
        self.caret_pos = max(self.caret_pos - 1, 0)

    def right_move_right(self):
        """ Moves right boundary of the text window right-wards.

        """
        if self.window_right <= len(self.text):
            self.window_right += 1

    def right_move_left(self):
        """ Moves right boundary of the text window left-wards.

        """
        if self.window_right > 0:
            self.window_right -= 1

    def left_move_right(self):
        """ Moves left boundary of the text window right-wards.

        """
        if self.window_left <= len(self.text):
            self.window_left += 1

    def left_move_left(self):
        """ Moves left boundary of the text window left-wards.

        """
        if self.window_left > 0:
            self.window_left -= 1

    def add_character(self, character):
        """ Inserts a character into the text and moves window and caret accordingly.

        Parameters
        ----------
        character : str

        """
        if len(character) > 1 and character.lower() != "space":
            return
        if character.lower() == "space":
            character = " "
        self.text = (self.text[:self.caret_pos] +
                     character +
                     self.text[self.caret_pos:])
        self.move_caret_right()
        if (self.window_right -
                self.window_left == self.height * self.width - 1):
            self.left_move_right()
        self.right_move_right()

    def remove_character(self):
        """ Removes a character from the text and moves window and caret accordingly.

        """
        if self.caret_pos == 0:
            return
        self.text = self.text[:self.caret_pos - 1] + self.text[self.caret_pos:]
        self.move_caret_left()
        if len(self.text) < self.height * self.width - 1:
            self.right_move_left()
        if (self.window_right -
                self.window_left == self.height * self.width - 1):
            if self.window_left > 0:
                self.left_move_left()
                self.right_move_left()

    def move_left(self):
        """ Handles left button press.

        """
        self.move_caret_left()
        if self.caret_pos == self.window_left - 1:
            if (self.window_right -
                    self.window_left == self.height * self.width - 1):
                self.left_move_left()
                self.right_move_left()

    def move_right(self):
        """ Handles right button press.

        """
        self.move_caret_right()
        if self.caret_pos == self.window_right + 1:
            if (self.window_right -
                    self.window_left == self.height * self.width - 1):
                self.left_move_right()
                self.right_move_right()

    def showable_text(self, show_caret):
        """ Chops out text to be shown on the screen.

        Parameters
        ----------
        show_caret : bool
            Whether or not to show the caret.

        """
        if show_caret:
            ret_text = (self.text[:self.caret_pos] +
                        "_" +
                        self.text[self.caret_pos:])
        else:
            ret_text = self.text
        ret_text = ret_text[self.window_left:self.window_right + 1]
        return ret_text

    def render_text(self, show_caret=True):
        """ Renders text after processing.

        Parameters
        ----------
        show_caret : bool
            Whether or not to show the caret.

        """
        text = self.showable_text(show_caret)
        if text == "":
            text = "Enter Text"
        self.actor.message = self.width_set_text(text)

    def edit_mode(self):
        """ Turns on edit mode.

        """
        if self.init:
            self.text = ""
            self.init = False
            self.caret_pos = 0
        self.render_text()

    def set_center(self, position):
        """ Sets the text center to position.

        Parameters
        ----------
        position : (float, float)

        """
        self.actor.position = position

    @staticmethod
    def left_button_press(i_ren, obj, textbox_object):
        """ Left button press handler for textbox

        Parameters
        ----------
        i_ren: :class:`CustomInteractorStyle`
        obj: :class:`vtkActor`
            The picked actor
        textbox_object: :class:`TextBox2D`

        """
        i_ren.add_active_prop(textbox_object.actor.get_actor())
        textbox_object.edit_mode()
        i_ren.force_render()

    @staticmethod
    def key_press(i_ren, obj, textbox_object):
        """ Key press handler for textbox

        Parameters
        ----------
        i_ren: :class:`CustomInteractorStyle`
        obj: :class:`vtkActor`
            The picked actor
        textbox_object: :class:`TextBox2D`

        """
        key = i_ren.event.key
        is_done = textbox_object.handle_character(key)
        if is_done:
            i_ren.remove_active_prop(textbox_object.actor.get_actor())

        i_ren.force_render()


class LineSlider2D(UI):
    """ A 2D Line Slider.

    A sliding ring on a line with a percentage indicator.

    Currently supports:
    - A disk on a line (a thin rectangle).
    - Setting disk position.

    Attributes
    ----------
    line_width : int
        Width of the line on which the disk will slide.
    inner_radius : int
        Inner radius of the disk (ring).
    outer_radius : int
        Outer radius of the disk.
    center : (float, float)
        Center of the slider.
    length : int
        Length of the slider.
    slider_line : :class:`vtkActor`
        The line on which the slider disk moves.
    slider_disk : :class:`vtkActor`
        The moving slider disk.
    text : :class:`TextBlock2D`
        The text that shows percentage.

    """
    def __init__(self, line_width=5, inner_radius=0, outer_radius=10,
                 center=(450, 300), length=200, initial_value=50,
                 min_value=0, max_value=100, text_size=16,
                 text_template="{value:.1f} ({ratio:.0%})"):
        """
        Parameters
        ----------
        line_width : int
            Width of the line on which the disk will slide.
        inner_radius : int
            Inner radius of the disk (ring).
        outer_radius : int
            Outer radius of the disk.
        center : (float, float)
            Center of the slider.
        length : int
            Length of the slider.
        initial_value : float
            Initial value of the slider.
        min_value : float
            Minimum value of the slider.
        max_value : float
            Maximum value of the slider.
        text_size : int
            Size of the text to display alongside the slider (pt).
        text_template : str, callable
            If str, text template can contain one or multiple of the
            replacement fields: `{value:}`, `{ratio:}`.
            If callable, this instance of `:class:LineSlider2D` will be
            passed as argument to the text template function.

        """
        super(LineSlider2D, self).__init__()

        self.length = length
        self.min_value = min_value
        self.max_value = max_value

        self.text_template = text_template

        self.line_width = line_width
        self.center = center
        self.current_state = center[0]
        self.left_x_position = center[0] - length / 2
        self.right_x_position = center[0] + length / 2
        self._ratio = (self.current_state - self.left_x_position) / length

        self.slider_line = None
        self.slider_disk = None
        self.text = None

        self.build_actors(inner_radius=inner_radius,
                          outer_radius=outer_radius, text_size=text_size)

        # Setting the disk position will also update everything.
        self.value = initial_value
        # self.update()

        self.handle_events(None)

    def build_actors(self, inner_radius, outer_radius, text_size):
        """ Builds required actors.

        Parameters
        ----------
        inner_radius: int
            The inner radius of the sliding disk.
        outer_radius: int
            The outer radius of the sliding disk.
        text_size: int
            Size of the text that displays percentage.

        """
        # Slider Line
        self.slider_line = Rectangle2D(size=(self.length, self.line_width),
                                       center=self.center).actor
        self.slider_line.GetProperty().SetColor(1, 0, 0)
        # /Slider Line

        # Slider Disk
        # Create source
        disk = vtk.vtkDiskSource()
        disk.SetInnerRadius(inner_radius)
        disk.SetOuterRadius(outer_radius)
        disk.SetRadialResolution(10)
        disk.SetCircumferentialResolution(50)
        disk.Update()

        # Mapper
        mapper = vtk.vtkPolyDataMapper2D()
        mapper.SetInputConnection(disk.GetOutputPort())

        # Actor
        self.slider_disk = vtk.vtkActor2D()
        self.slider_disk.SetMapper(mapper)
        # /Slider Disk

        # Slider Text
        self.text = TextBlock2D()
        self.text.position = (self.left_x_position - 50, self.center[1] - 10)
        self.text.font_size = text_size
        # /Slider Text

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        return [self.slider_line, self.slider_disk, self.text.get_actor()]

    def set_position(self, position):
        """ Sets the disk's position.

        Parameters
        ----------
        position : (float, float)
            The absolute position of the disk (x, y).

        """
        x_position = position[0]

        if x_position < self.center[0] - self.length/2:
            x_position = self.center[0] - self.length/2

        if x_position > self.center[0] + self.length/2:
            x_position = self.center[0] + self.length/2

        self.current_state = x_position
        self.update()

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        value_range = self.max_value - self.min_value
        self.ratio = (value - self.min_value) / value_range

    @property
    def ratio(self):
        return self._ratio

    @ratio.setter
    def ratio(self, ratio):
        position_x = self.left_x_position + ratio*self.length
        self.set_position((position_x, None))

    def format_text(self):
        """ Returns formatted text to display along the slider. """
        if callable(self.text_template):
            return self.text_template(self)

        return self.text_template.format(ratio=self.ratio, value=self.value)

    def update(self):
        """ Updates the slider. """

        # Compute the ratio determined by the position of the slider disk.
        length = float(self.right_x_position - self.left_x_position)
        assert length == self.length
        self._ratio = (self.current_state - self.left_x_position) / length

        # Compute the selected value considering min_value and max_value.
        value_range = self.max_value - self.min_value
        self._value = self.min_value + self.ratio*value_range

        # Update text disk actor.
        self.slider_disk.SetPosition(self.current_state, self.center[1])

        # Update text.
        text = self.format_text()
        self.text.message = text
        offset_x = 8 * len(text) / 2.
        offset_y = 30
        self.text.position = (self.current_state - offset_x,
                              self.center[1] - offset_y)

    def set_center(self, position):
        """ Sets the center of the slider to position.

        Parameters
        ----------
        position : (float, float)
            The new center of the whole slider (x, y).

        """
        self.slider_line.SetPosition(position[0] - self.length / 2,
                                     position[1] - self.line_width / 2)

        x_change = position[0] - self.center[0]
        self.current_state += x_change
        self.center = position
        self.left_x_position = position[0] - self.length / 2
        self.right_x_position = position[0] + self.length / 2
        self.set_position((self.current_state, self.center[1]))

    @staticmethod
    def line_click_callback(i_ren, obj, slider):
        """ Update disk position and grab the focus.

        Parameters
        ----------
        i_ren : :class:`CustomInteractorStyle`
        obj : :class:`vtkActor`
            The picked actor
        slider : :class:`LineSlider2D`

        """
        position = i_ren.event.position
        slider.set_position(position)
        i_ren.force_render()
        i_ren.event.abort()  # Stop propagating the event.

    @staticmethod
    def disk_press_callback(i_ren, obj, slider):
        """ Only need to grab the focus.

        Parameters
        ----------
        i_ren : :class:`CustomInteractorStyle`
        obj : :class:`vtkActor`
            The picked actor
        slider : :class:`LineSlider2D`

        """
        i_ren.event.abort()  # Stop propagating the event.

    @staticmethod
    def disk_move_callback(i_ren, obj, slider):
        """ Actual disk movement.

        Parameters
        ----------
        i_ren : :class:`CustomInteractorStyle`
        obj : :class:`vtkActor`
            The picked actor
        slider : :class:`LineSlider2D`

        """
        position = i_ren.event.position
        slider.set_position(position)
        i_ren.force_render()
        i_ren.event.abort()  # Stop propagating the event.

    def handle_events(self, actor):
        """ Handle all events for the LineSlider.
        Base method needs to be overridden due to multiple actors.

        """
        self.add_callback(self.slider_line, "LeftButtonPressEvent",
                          self.line_click_callback, 1)
        self.add_callback(self.slider_disk, "LeftButtonPressEvent",
                          self.disk_press_callback)
        self.add_callback(self.slider_disk, "MouseMoveEvent",
                          self.disk_move_callback)
        self.add_callback(self.slider_line, "MouseMoveEvent",
                          self.disk_move_callback)


class DiskSlider2D(UI):
    """ A disk slider.

    A disk moves along the boundary of a ring.
    Goes from 0-360 degrees.

    Attributes
    ----------
    base_disk_center: (float, float)
        Position of the system.
    slider_inner_radius: int
        Inner radius of the base disk.
    slider_outer_radius: int
        Outer radius of the base disk.
    slider_radius: float
        Average radius of the base disk.
    handle_outer_radius: int
        Outer radius of the slider's handle.
    handle_inner_radius: int
        Inner radius of the slider's handle.
    previous_value: float
        Value of Rotation of the actor before the current value.
    initial_value: float
        Initial Value of Rotation of the actor assigned on creation of object.

    """
    def __init__(self, position=(0, 0),
                 initial_value=180, min_value=0, max_value=360,
                 slider_inner_radius=40, slider_outer_radius=44,
                 handle_inner_radius=10, handle_outer_radius=0,
                 text_size=16,
                 text_template="{ratio:.0%}"):

        """
        Parameters
        ----------
        position : (float, float)
            Position (x, y) of the slider's center.
        initial_value : float
            Initial value of the slider.
        min_value : float
            Minimum value of the slider.
        max_value : float
            Maximum value of the slider.
        slider_inner_radius : int
            Inner radius of the base disk.
        slider_outer_radius : int
            Outer radius of the base disk.
        handle_outer_radius : int
            Outer radius of the slider's handle.
        handle_inner_radius : int
            Inner radius of the slider's handle.
        text_size : int
            Size of the text to display alongside the slider (pt).
        text_template : str, callable
            If str, text template can contain one or multiple of the
            replacement fields: `{value:}`, `{ratio:}`, `{angle:}`.
            If callable, this instance of `:class:DiskSlider2D` will be
            passed as argument to the text template function.

        """
        super(DiskSlider2D, self).__init__()
        self.center = np.array(position)
        self.min_value = min_value
        self.max_value = max_value
        self.initial_value = initial_value
        self.slider_inner_radius = slider_inner_radius
        self.slider_outer_radius = slider_outer_radius
        self.handle_inner_radius = handle_inner_radius
        self.handle_outer_radius = handle_outer_radius
        self.slider_radius = (slider_inner_radius + slider_outer_radius) / 2.

        self.handle = None
        self.base_disk = None

        self.text = None
        self.text_size = text_size
        self.text_template = text_template

        self.build_actors()

        # By setting the value, it also updates everything.
        self.value = initial_value
        self.previous_value = initial_value
        self.handle_events(None)

    def build_actors(self):
        """ Builds actors for the system.

        """
        base_disk = vtk.vtkDiskSource()
        base_disk.SetInnerRadius(self.slider_inner_radius)
        base_disk.SetOuterRadius(self.slider_outer_radius)
        base_disk.SetRadialResolution(10)
        base_disk.SetCircumferentialResolution(50)
        base_disk.Update()

        base_disk_mapper = vtk.vtkPolyDataMapper2D()
        base_disk_mapper.SetInputConnection(base_disk.GetOutputPort())

        self.base_disk = vtk.vtkActor2D()
        self.base_disk.SetMapper(base_disk_mapper)
        self.base_disk.GetProperty().SetColor(1, 0, 0)
        self.base_disk.SetPosition(self.center)

        handle = vtk.vtkDiskSource()
        handle.SetInnerRadius(self.handle_inner_radius)
        handle.SetOuterRadius(self.handle_outer_radius)
        handle.SetRadialResolution(10)
        handle.SetCircumferentialResolution(50)
        handle.Update()

        handle_mapper = vtk.vtkPolyDataMapper2D()
        handle_mapper.SetInputConnection(handle.GetOutputPort())

        self.handle = vtk.vtkActor2D()
        self.handle.SetMapper(handle_mapper)

        self.text = TextBlock2D()
        offset = np.array((16., 8.))
        self.text.position = self.center - offset
        self.text.font_size = self.text_size

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        value_range = self.max_value - self.min_value
        self.ratio = (value - self.min_value) / value_range

    @property
    def previous_value(self):
        return self._previous_value

    @previous_value.setter
    def previous_value(self, previous_value):
        self._previous_value = previous_value

    @property
    def ratio(self):
        return self._ratio

    @ratio.setter
    def ratio(self, ratio):
        self.angle = ratio * TWO_PI

    @property
    def angle(self):
        """ Angle (in rad) the handle makes with x-axis """
        return self._angle

    @angle.setter
    def angle(self, angle):
        self._angle = angle % TWO_PI  # Wraparound
        self.update()

    def format_text(self):
        """ Returns formatted text to display along the slider. """
        if callable(self.text_template):
            return self.text_template(self)

        return self.text_template.format(ratio=self.ratio, value=self.value,
                                         angle=np.rad2deg(self.angle))

    def update(self):
        """ Updates the slider. """

        # Compute the ratio determined by the position of the slider disk.
        self._ratio = self.angle / TWO_PI

        # Compute the selected value considering min_value and max_value.
        value_range = self.max_value - self.min_value
        try:
            self._previous_value = self.value
        except:
            self._previous_value = self.initial_value
        self._value = self.min_value + self.ratio*value_range

        # Update text disk actor.
        x = self.slider_radius * np.cos(self.angle) + self.center[0]
        y = self.slider_radius * np.sin(self.angle) + self.center[1]
        self.handle.SetPosition(x, y)

        # Update text.
        text = self.format_text()
        self.text.message = text

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        return [self.base_disk, self.handle, self.text.get_actor()]

    def move_handle(self, click_position):
        """Moves the slider's handle.

        Parameters
        ----------
        click_position: (float, float)
            Position of the mouse click.

        """
        x, y = np.array(click_position) - self.center
        angle = np.arctan2(y, x)
        if angle < 0:
            angle += TWO_PI

        self.angle = angle

    def set_center(self, position):
        """ Changes the slider's center position.

        Parameters
        ----------
        position : (float, float)
            New position (x, y).

        """
        position = np.array(position)
        offset = position - self.center
        self.base_disk.SetPosition(position)
        self.handle.SetPosition(*(offset + self.handle.GetPosition()))
        self.text.position += offset
        self.center = position

    @staticmethod
    def base_disk_click_callback(i_ren, obj, slider):
        """ Update disk position and grab the focus.

        Parameters
        ----------
        i_ren : :class:`CustomInteractorStyle`
        obj : :class:`vtkActor`
            The picked actor
        slider : :class:`DiskSlider2D`

        """
        click_position = i_ren.event.position
        slider.move_handle(click_position=click_position)
        i_ren.force_render()
        i_ren.event.abort()  # Stop propagating the event.

    @staticmethod
    def handle_move_callback(i_ren, obj, slider):
        """ Move the slider's handle.

        Parameters
        ----------
        i_ren : :class:`CustomInteractorStyle`
        obj : :class:`vtkActor`
            The picked actor
        slider : :class:`DiskSlider2D`

        """
        click_position = i_ren.event.position
        slider.move_handle(click_position=click_position)
        i_ren.force_render()
        i_ren.event.abort()  # Stop propagating the event.

    @staticmethod
    def handle_press_callback(i_ren, obj, slider):
        """ This is only needed to grab the focus.

        Parameters
        ----------
        i_ren : :class:`CustomInteractorStyle`
        obj : :class:`vtkActor`
            The picked actor
        slider : :class:`DiskSlider2D`

        """
        i_ren.event.abort()  # Stop propagating the event.

    def handle_events(self, actor):
        """ Handle all default slider events.

        """
        self.add_callback(self.base_disk, "LeftButtonPressEvent",
                          self.base_disk_click_callback, 1)
        self.add_callback(self.handle, "LeftButtonPressEvent",
                          self.handle_press_callback)
        self.add_callback(self.base_disk, "MouseMoveEvent",
                          self.handle_move_callback)
        self.add_callback(self.handle, "MouseMoveEvent",
                          self.handle_move_callback)


class FileSelectMenu2D(UI):
    """ A menu to select files in the current folder.

    Can go to new folder, previous folder and select a file
    and keep it in a variable.

    Attributes
    ----------
    n_text_actors: int
        The number of text actors. Calculated dynamically.
    selected_file: string
        Current selected file.
    text_item_list: list(:class:`FileSelectMenuText2D`)
        List of FileSelectMenuText2Ds - both visible and invisible.
    window_offset: int
        Used for scrolling.
        Tells you the index of the first visible FileSelectMenuText2D
        object.
    size: (float, float)
        The size of the system (x, y) in pixels.
    font_size: int
        The font size in pixels.
    line_spacing: float
        Distance between menu text items in pixels.
    parent_ui: :class:`UI`
        The UI component this object belongs to.
    extensions: list(string)
            List of extensions to be shown as files.

    """

    def __init__(self, size, font_size, position, parent, extensions,
                 directory_path, reverse_scrolling=False, line_spacing=1.4):
        """
        Parameters
        ----------
        size: (float, float)
            The size of the system (x, y) in pixels.
        font_size: int
            The font size in pixels.
        parent: :class:`UI`
            The UI component this object belongs to.
            This will be useful when this UI element is used as a
            part of other UI elements, like a file save dialog.
        position: (float, float)
            The initial position (x, y) in pixels.
        reverse_scrolling: {True, False}
            If True, scrolling up will move the list of files down.
        line_spacing: float
            Distance between menu text items in pixels.
        extensions: list(string)
            List of extensions to be shown as files.
        directory_path: string
            Path of the directory where this dialog should open.
            Example: os.getcwd()

        """
        super(FileSelectMenu2D, self).__init__()

        self.size = size
        self.font_size = font_size
        self.parent_ui = parent
        self.reverse_scrolling = reverse_scrolling
        self.line_spacing = line_spacing
        self.extensions = extensions

        self.n_text_actors = 0  # Initialisation Value
        self.text_item_list = []
        self.selected_file = ""
        self.window_offset = 0
        self.current_directory = directory_path
        self.buttons = dict()

        self.menu = self.build_actors(position)

        self.fill_text_actors()
        self.handle_events(None)

    def add_to_renderer(self, ren):
        self.menu.add_to_renderer(ren)
        super(FileSelectMenu2D, self).add_to_renderer(ren)
        for menu_text in self.text_item_list:
            menu_text.add_to_renderer(ren)

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        return [self.buttons["up"], self.buttons["down"]]

    def build_actors(self, position):
        """ Builds the number of text actors that will fit in the given size.

        Allots them positions in the panel, which is only there to allot positions,
        otherwise the panel itself is invisible.

        Parameters
        ----------
        position: (float, float)
            Position of the panel (x, y) in pixels.

        """
        # Calculating the number of text actors.
        self.n_text_actors = int(self.size[1]/(self.font_size*self.line_spacing))

        # This panel is just to facilitate the addition of actors at the right positions
        panel = Panel2D(center=position, size=self.size, color=(1, 1, 1))

        # Initialisation of empty text actors
        for i in range(self.n_text_actors):

            text = FileSelectMenuText2D(position=(0, 0), font_size=self.font_size,
                                        file_select=self)
            text.parent_UI = self.parent_ui
            self.ui_list.append(text)
            self.text_item_list.append(text)

            panel.add_element(text, 'relative',
                              (0.1,
                               float(self.n_text_actors-i - 1) /
                               float(self.n_text_actors)))

        up_button = Button2D({"up": read_viz_icons(fname="arrow-up.png")})
        panel.add_element(up_button, 'relative', (0.95, 0.95))
        self.buttons["up"] = up_button

        down_button = Button2D({"down": read_viz_icons(fname="arrow-down.png")})
        panel.add_element(down_button, 'relative', (0.95, 0.05))
        self.buttons["down"] = down_button

        return panel

    @staticmethod
    def up_button_callback(i_ren, obj, file_select_menu):
        """ Pressing up button scrolls up in the menu.

        Parameters
        ----------
        i_ren: :class:`CustomInteractorStyle`
        obj: :class:`vtkActor`
            The picked actor
        file_select_menu: :class:`FileSelectMenu2D`

        """
        all_file_names = file_select_menu.get_all_file_names()

        if (file_select_menu.n_text_actors +
                file_select_menu.window_offset) <= len(all_file_names):
            if file_select_menu.window_offset > 0:
                file_select_menu.window_offset -= 1
                file_select_menu.fill_text_actors()

        i_ren.force_render()
        i_ren.event.abort()  # Stop propagating the event.

    @staticmethod
    def down_button_callback(i_ren, obj, file_select_menu):
        """ Pressing down button scrolls down in the menu.

        Parameters
        ----------
        i_ren: :class:`CustomInteractorStyle`
        obj: :class:`vtkActor`
            The picked actor
        file_select_menu: :class:`FileSelectMenu2D`

        """
        all_file_names = file_select_menu.get_all_file_names()

        if (file_select_menu.n_text_actors +
                file_select_menu.window_offset) < len(all_file_names):
            file_select_menu.window_offset += 1
            file_select_menu.fill_text_actors()

        i_ren.force_render()
        i_ren.event.abort()  # Stop propagating the event.

    def fill_text_actors(self):
        """ Fills file/folder names to text actors.

        The list is truncated if the number of file/folder names is greater
        than the available number of text actors.

        """
        # Flush all the text actors
        for text_item in self.text_item_list:
            text_item.text_actor.message = ""
            text_item.text_actor.actor.SetVisibility(False)

        all_file_names = self.get_all_file_names()

        clipped_file_names = all_file_names[self.window_offset:self.n_text_actors + self.window_offset]

        # Allot file names as in the above list
        i = 0
        for file_name in clipped_file_names:
            self.text_item_list[i].text_actor.actor.SetVisibility(True)
            self.text_item_list[i].set_attributes(file_name[0], file_name[1])
            if file_name[0] == self.selected_file:
                self.text_item_list[i].mark_selected()
            i += 1

    def get_all_file_names(self):
        """ Gets file and directory names.

        Returns
        -------
        all_file_names: list(string)
            List of all file and directory names as string.

        """
        all_file_names = []

        directory_names = self.get_directory_names()
        for directory_name in directory_names:
            all_file_names.append((directory_name, "directory"))

        file_names = self.get_file_names()
        for file_name in file_names:
            all_file_names.append((file_name, "file"))

        return all_file_names

    def get_directory_names(self):
        """ Re-allots file names to the text actors.

        Uses FileSelectMenuText2D for selecting files and folders.

        Returns
        -------
        directory_names: list(string)
            List of all directory names as string.

        """
        # A list of directory names in the current directory
        directory_names = next(os.walk(self.current_directory))[1]
        directory_names = [os.path.basename(os.path.abspath(dn)) for dn in directory_names]
        directory_names = ["../"] + directory_names

        return directory_names

    def get_file_names(self):
        """ Re-allots file names to the text actors.

        Uses FileSelectMenuText2D for selecting files and folders.

        Returns
        -------
        file_names: list(string)
            List of all file names as string.

        """
        # A list of file names with extension in the current directory
        file_names = []
        for extension in self.extensions:
            file_names += glob.glob(self.current_directory + "/*." + extension)
        file_names = [os.path.basename(os.path.abspath(fn)) for fn in file_names]
        return file_names

    def select_file(self, file_name):
        """ Changes the selected file name.

        Parameters
        ----------
        file_name: string
            Name of the file.

        """
        self.selected_file = file_name

    def set_center(self, position):
        """ Sets the elements center.

        Parameters
        ----------
        position: (float, float)
            New position (x, y) in pixels.

        """
        self.menu.set_center(position=position)

    def handle_events(self, actor):
        self.add_callback(self.buttons["up"].actor, "LeftButtonPressEvent",
                          self.up_button_callback)
        self.add_callback(self.buttons["down"].actor, "LeftButtonPressEvent",
                          self.down_button_callback)

        # Handle mouse wheel events
        up_event = "MouseWheelForwardEvent"
        down_event = "MouseWheelBackwardEvent"
        if self.reverse_scrolling:
            up_event, down_event = down_event, up_event  # Swap events

        self.add_callback(self.menu.get_actors()[0], up_event,
                          self.up_button_callback)
        self.add_callback(self.menu.get_actors()[0], down_event,
                          self.down_button_callback)

        for text_ui in self.text_item_list:
            self.add_callback(text_ui.text_actor.get_actors()[0], up_event,
                              self.up_button_callback)
            self.add_callback(text_ui.text_actor.get_actors()[0], down_event,
                              self.down_button_callback)


class FileSelectMenuText2D(UI):
    """ The text to select folder in a file select menu.

    Provides a callback to change the directory.

    Attributes
    ----------
    file_name: string
        The name of the file the text is displaying.
    file_type: string
        Whether the file is a file or directory.
    file_select: :class:`FileSelect2D`
        The FileSelectMenu2D reference this text belongs to.

    """

    def __init__(self, font_size, position, file_select):
        """
        Parameters
        ----------
        font_size: int
            The font size of the text in pixels.
        position: (float, float)
            Absolute text position (x, y) in pixels.
        file_select: :class:`FileSelect2D`
            The FileSelectMenu2D reference this text belongs to.

        """
        super(FileSelectMenuText2D, self).__init__()

        self.file_name = ""
        self.file_type = ""
        self.file_select = file_select

        self.text_actor = self.build_actor(position=position, font_size=font_size)

        self.handle_events(self.text_actor.get_actor())

        self.on_left_mouse_button_clicked = self.left_button_clicked

    def build_actor(self, position, text="Text", color=(1, 1, 1), font_family='Arial',
                    justification='left', bold=False, italic=False,
                    shadow=False, font_size='14'):
        """ Builds a text actor.

        Parameters
        ----------
        text: string
            The initial text while building the actor.
        position: (float, float)
            The text position (x, y) in pixels.
        color: (float, float, float)
            Values must be between 0-1 (RGB).
        font_family: string
            Currently only supports Arial.
        justification: string
            Text justification - left, right or center.
        bold: bool
            Whether or not the text is bold.
        italic: bool
            Whether or not the text is italicized.
        shadow: bool
            Whether or not the text has shadow.
        font_size: int
            The font size of the text in pixels.

        Returns
        -------
        text_actor: :class:`TextBlock2D`
            The base text actor.

        """
        text_actor = TextBlock2D()
        text_actor.position = position
        text_actor.message = text
        text_actor.font_size = font_size
        text_actor.font_family = font_family
        text_actor.justification = justification
        text_actor.bold = bold
        text_actor.italic = italic
        text_actor.shadow = shadow
        text_actor.color = color

        if vtk.vtkVersion.GetVTKVersion() <= "6.2.0":
            pass
        else:
            text_actor.actor.GetTextProperty().SetBackgroundColor(1, 1, 1)
            text_actor.actor.GetTextProperty().SetBackgroundOpacity(1.0)

        text_actor.actor.GetTextProperty().SetColor(0, 0, 0)
        text_actor.actor.GetTextProperty().SetLineSpacing(1)

        return text_actor

    def get_actors(self):
        """ Returns the actors that compose this UI component.

        """
        return [self.text_actor.get_actor()]

    def set_attributes(self, file_name, file_type):
        """  Set attributes (file name and type) of this component.

        This function is for use by a FileSelectMenu2D to set the
        current file_name and file_type for this FileSelectMenuText2D
        component.

        Parameters
        ----------
        file_name: string
            The name of the file.
        file_type: string
            File type = directory or file.

        """
        self.file_name = file_name
        self.file_type = file_type
        self.text_actor.message = file_name

        if vtk.vtkVersion.GetVTKVersion() <= "6.2.0":
            self.text_actor.get_actor().GetTextProperty().SetColor(1, 1, 1)
            if file_type != "file":
                self.text_actor.get_actor().GetTextProperty().SetBold(True)

        else:
            if file_type == "file":
                self.text_actor.get_actor().GetTextProperty().SetBackgroundColor(0, 0, 0)
                self.text_actor.get_actor().GetTextProperty().SetColor(1, 1, 1)
            else:
                self.text_actor.get_actor().GetTextProperty().SetBackgroundColor(1, 1, 1)
                self.text_actor.get_actor().GetTextProperty().SetColor(0, 0, 0)

    def mark_selected(self):
        """ Changes the background color of the actor.

        """
        if vtk.vtkVersion.GetVTKVersion() <= "6.2.0":
            self.text_actor.actor.GetTextProperty().SetColor(1, 0, 0)
        else:
            self.text_actor.actor.GetTextProperty().SetBackgroundColor(1, 0, 0)
            self.text_actor.actor.GetTextProperty().SetBackgroundOpacity(1.0)

    @staticmethod
    def left_button_clicked(i_ren, obj, file_select_text):
        """ A callback to handle left click for this UI element.

        Parameters
        ----------
        i_ren: :class:`CustomInteractorStyle`
        obj: :class:`vtkActor`
            The picked actor
        file_select_text: :class:`FileSelectMenuText2D`

        """

        if file_select_text.file_type == "directory":
            file_select_text.file_select.select_file(file_name="")
            file_select_text.file_select.window_offset = 0
            file_select_text.file_select.current_directory = os.path.abspath(
                os.path.join(file_select_text.file_select.current_directory,
                             file_select_text.text_actor.message))
            file_select_text.file_select.window = 0
            file_select_text.file_select.fill_text_actors()
        else:
            file_select_text.file_select.select_file(
                file_name=file_select_text.file_name)
            file_select_text.file_select.fill_text_actors()
            file_select_text.mark_selected()

        i_ren.force_render()
        i_ren.event.abort()  # Stop propagating the event.

    def set_center(self, position):
        """ Sets the text center to position.

        Parameters
        ----------
        position: (float, float)
            The new position (x, y) in pixels.
        """
        self.text_actor.position = position



#!/usr/bin/env python
from __future__ import division

import logging
import numpy as np

from nibabel.streamlines import save, Tractogram

from dipy.direction import DeterministicMaximumDirectionGetter
from dipy.io.image import load_nifti
from dipy.io.peaks import load_peaks
from dipy.tracking import utils
from dipy.tracking.local import (ThresholdTissueClassifier,
                                 LocalTracking)
from dipy.workflows.workflow import Workflow


class GenericTrackFlow(Workflow):
    def _core_run(self, stopping_path, stopping_thr, seeding_path,
                  seed_density, use_sh, pam, out_tract):

        stop, affine = load_nifti(stopping_path)
        classifier = ThresholdTissueClassifier(stop, stopping_thr)
        logging.info('classifier done')
        seed_mask, _ = load_nifti(seeding_path)
        seeds = \
            utils.seeds_from_mask(
                seed_mask,
                density=[seed_density, seed_density, seed_density],
                affine=affine)
        logging.info('seeds done')
        direction_getter = pam

        if use_sh:
            direction_getter = \
                DeterministicMaximumDirectionGetter.from_shcoeff(
                    pam.shm_coeff,
                    max_angle=30.,
                    sphere=pam.sphere)

        streamlines = LocalTracking(direction_getter, classifier,
                                    seeds, affine, step_size=.5)
        logging.info('LocalTracking initiated')

        tractogram = Tractogram(streamlines, affine_to_rasmm=np.eye(4))
        save(tractogram, out_tract)

        logging.info('Saved {0}'.format(out_tract))


class DetTrackPAMFlow(GenericTrackFlow):
    @classmethod
    def get_short_name(cls):
        return 'det_track'

    def run(self, pam_files, stopping_files, seeding_files,
            stopping_thr=0.2,
            seed_density=1,
            use_sh=False,
            out_dir='',
            out_tractogram='tractogram.trk'):

        """ Workflow for deterministic using a saved peaks and metrics (PAM)
        file as input.

        Parameters
        ----------
        pam_files : string
           Path to the peaks and metrics files. This path may contain
            wildcards to use multiple masks at once.
        stopping_files : string
            Path of FA or other images used for stopping criteria for tracking.
        seeding_files : string
            A binary image showing where we need to seed for tracking.
        stopping_thr : float, optional
            Threshold applied to stopping volume's data to identify where
             tracking has to stop (default 0.25).
        seed_density : int, optional
            Number of seeds per dimension inside voxel (default 1).
             For example, seed_density of 2 means 8 regularly distributed points
             in the voxel. And seed density of 1 means 1 point at the center
             of the voxel.
        use_sh : bool, optional
            Use spherical harmonics saved in peaks to find the
             maximum peak cone. (default False)
        out_dir : string, optional
           Output directory (default input file directory)
        out_tractogram : string, optional
           Name of the tractogram file to be saved (default 'tractogram.trk')

        References
        ----------
        Garyfallidis, University of Cambridge, PhD thesis 2012.
        Amirbekian, University of California San Francisco, PhD thesis 2017.
        """
        io_it = self.get_io_iterator()

        for pams_path, stopping_path, seeding_path, out_tract in io_it:

            logging.info('Deterministic tracking on {0}'
                         .format(pams_path))

            pam = load_peaks(pams_path, verbose=False)

            self._core_run(stopping_path, stopping_thr, seeding_path,
                           seed_density, use_sh, pam, out_tract)




"""

===========================
Read/Write streamline files
===========================

Overview
========

DIPY_ can read and write many different file formats. In this example
we give a short introduction on how to use it for loading or saving streamlines.

Read :ref:`faq`

"""

import numpy as np
from dipy.data import get_data
from dipy.io.streamline import load_trk, save_trk
from dipy.tracking.streamline import Streamlines

"""
1. Read/write streamline files with DIPY.
"""

fname = get_data('fornix')
print(fname)

# Read Streamlines
streams, hdr = load_trk(fname)
streamlines = Streamlines(streams)

# Save Streamlines
save_trk("my_streamlines.trk", streamlines=streamlines, affine=np.eye(4))


"""
2. We also work on our HDF5 based file format which can read/write massive datasets
   (as big as the size of you free disk space). With `Dpy` we can support

  * direct indexing from the disk
  * memory usage always low
  * extensions to include different arrays in the same file

Here is a simple example.
"""

from dipy.io.dpy import Dpy
dpw = Dpy('fornix.dpy', 'w')

"""
Write many streamlines at once.
"""

dpw.write_tracks(streamlines)

"""
Write one track
"""

dpw.write_track(streamlines[0])

"""
or one track each time.
"""

for t in streamlines:
    dpw.write_track(t)

dpw.close()

"""
Read streamlines directly from the disk using their indices

.. include:: ../links_names.inc
"""

dpr = Dpy('fornix.dpy', 'r')
some_streamlines = dpr.read_tracksi([0, 10, 20, 30, 100])
dpr.close()


print(len(streamlines))
print(len(some_streamlines))



import numpy as np
import nibabel as nib
from numpy.testing import assert_equal, run_module_suite
from dipy.data import get_data
from dipy.segment.bundles import RecoBundles
from dipy.tracking.distances import bundles_distances_mam
from dipy.tracking.streamline import Streamlines
from dipy.segment.clustering import qbx_and_merge


streams, hdr = nib.trackvis.read(get_data('fornix'))
fornix = [s[0] for s in streams]

f = Streamlines(fornix)
f1 = f.copy()

f2 = f1[:20].copy()
f2._data += np.array([50, 0, 0])

f3 = f1[200:].copy()
f3._data += np.array([100, 0, 0])

f.extend(f2)
f.extend(f3)


def test_rb_check_defaults():

    rb = RecoBundles(f, clust_thr=10)
    rec_trans, rec_labels, recognized = rb.recognize(model_bundle=f2,
                                                     model_clust_thr=5.,
                                                     reduction_thr=10)
    D = bundles_distances_mam(f2, recognized)

    # check if the bundle is recognized correctly
    for row in D:
        assert_equal(row.min(), 0)


def test_rb_disable_slr():

    rb = RecoBundles(f, clust_thr=10)

    rec_trans, rec_labels, recognized = rb.recognize(model_bundle=f2,
                                                     model_clust_thr=5.,
                                                     reduction_thr=10,
                                                     slr=False)

    D = bundles_distances_mam(f2, recognized)

    # check if the bundle is recognized correctly
    for row in D:
        assert_equal(row.min(), 0)


def test_rb_no_verbose_and_mam():

    rb = RecoBundles(f, clust_thr=10, verbose=False)

    rec_trans, rec_labels, recognized = rb.recognize(model_bundle=f2,
                                                     model_clust_thr=5.,
                                                     reduction_thr=10,
                                                     slr=True,
                                                     pruning_distance='mam')

    D = bundles_distances_mam(f2, recognized)

    # check if the bundle is recognized correctly
    for row in D:
        assert_equal(row.min(), 0)


def test_rb_clustermap():

    cluster_map = qbx_and_merge(f, thresholds=[40, 25, 20, 10])

    rb = RecoBundles(f, cluster_map=cluster_map, clust_thr=10)
    rec_trans, rec_labels, recognized = rb.recognize(model_bundle=f2,
                                                     model_clust_thr=5.,
                                                     reduction_thr=10)
    D = bundles_distances_mam(f2, recognized)

    # check if the bundle is recognized correctly
    for row in D:
        assert_equal(row.min(), 0)


def test_rb_no_neighb():
    # what if no neighbors are found? No recognition

    b = Streamlines(fornix)
    b1 = b.copy()

    b2 = b1[:20].copy()
    b2._data += np.array([100, 0, 0])

    b3 = b1[:20].copy()
    b3._data += np.array([300, 0, 0])

    b.extend(b3)

    rb = RecoBundles(b, clust_thr=10)
    rec_trans, rec_labels, recognized = rb.recognize(model_bundle=b2,
                                                     model_clust_thr=5.,
                                                     reduction_thr=10)

    assert_equal(len(recognized), 0)
    assert_equal(len(rec_labels), 0)
    assert_equal(len(rec_trans), 0)


def test_rb_reduction_mam():

    rb = RecoBundles(f, clust_thr=10, verbose=True)

    rec_trans, rec_labels, recognized = rb.recognize(model_bundle=f2,
                                                     model_clust_thr=5.,
                                                     reduction_thr=10,
                                                     reduction_distance='mam',
                                                     slr=True,
                                                     slr_metric='asymmetric',
                                                     pruning_distance='mam')

    D = bundles_distances_mam(f2, recognized)

    # check if the bundle is recognized correctly
    for row in D:
        assert_equal(row.min(), 0)


if __name__ == '__main__':

    run_module_suite()



import numpy as np
import numpy.testing as npt

import nibabel as nib
from nibabel.tmpdirs import TemporaryDirectory

from dipy.data import get_data
from dipy.workflows.align import ResliceFlow


def test_reslice():
    with TemporaryDirectory() as out_dir:
        data_path, _, _ = get_data('small_25')
        vol_img = nib.load(data_path)
        volume = vol_img.get_data()

        reslice_flow = ResliceFlow()
        reslice_flow.run(data_path, [1.5, 1.5, 1.5], out_dir=out_dir)

        out_path = reslice_flow.last_generated_outputs['out_resliced']
        out_img = nib.load(out_path)
        resliced = out_img.get_data()
        
        npt.assert_equal(resliced.shape[0] > volume.shape[0], True)
        npt.assert_equal(resliced.shape[1] > volume.shape[1], True)
        npt.assert_equal(resliced.shape[2] > volume.shape[2], True)
        npt.assert_equal(resliced.shape[-1], volume.shape[-1])
        

if __name__ == '__main__':
    npt.run_module_suite()



"""
==============================================================
Denoise images using Adaptive Soft Coefficient Matching (ASCM)
==============================================================

The adaptive soft coefficient matching (ASCM) as described in [Coupe11]_ is a
improved extension of non-local means (NLMEANS) denoising. ASCM gives a better
denoised images from two standard non-local means denoised versions of the
original data with different degrees sharpness. Here, one denoised input is
more "smooth" than the other (the easiest way to achieve this denoising is use
``non_local_means`` with two different patch radii).

ASCM involves these basic steps

* Computes wavelet decomposition of the noisy as well as denoised inputs

* Combines the wavelets for the output image in a way that it takes it's
  smoothness (low frequency components) from the input with larger smoothing,
  and the sharp features (high frequency components) from the input with
  less smoothing.

This way ASCM gives us a well denoised output while preserving the sharpness
of the image features.

Let us load the necessary modules
"""

import numpy as np
import matplotlib.pyplot as plt
import nibabel as nib
from dipy.data import (fetch_sherbrooke_3shell,
                       read_sherbrooke_3shell)
from dipy.denoise.noise_estimate import estimate_sigma
from time import time
from dipy.denoise.non_local_means import non_local_means
from dipy.denoise.adaptive_soft_matching import adaptive_soft_matching

"""
Choose one of the data from the datasets in dipy_
"""

fetch_sherbrooke_3shell()
img, gtab = read_sherbrooke_3shell()

data = img.get_data()
affine = img.affine

mask = data[..., 0] > 80
data = data[..., 1]

print("vol size", data.shape)

t = time()

"""
In order to generate the two pre-denoised versions of the data we will use the
``non_local_means`` denoising. For ``non_local_means`` first we need to
estimate the standard deviation of the noise. We use N=4 since the Sherbrooke
dataset was acquired on a 1.5T Siemens scanner with a 4 array head coil.
"""

sigma = estimate_sigma(data, N=4)

"""
For the denoised version of the original data which preserves sharper features,
we perform non-local means with smaller patch size.
"""

den_small = non_local_means(
    data,
    sigma=sigma,
    mask=mask,
    patch_radius=1,
    block_radius=1,
    rician=True)

"""
For the denoised version of the original data that implies more smoothing, we
perform non-local means with larger patch size.
"""

den_large = non_local_means(
    data,
    sigma=sigma,
    mask=mask,
    patch_radius=2,
    block_radius=1,
    rician=True)

"""
Now we perform the adaptive soft coefficient matching. Empirically we set the
adaptive parameter in ascm to be the average of the local noise variance,
in this case the sigma itself.
"""

den_final = adaptive_soft_matching(data, den_small, den_large, sigma[0])

print("total time", time() - t)

"""
To access the quality of this denoising procedure, we plot the an axial slice
of the original data, it's denoised output and residuals.
"""

axial_middle = data.shape[2] // 2

original = data[:, :, axial_middle].T
final_output = den_final[:, :, axial_middle].T
difference = np.abs(final_output.astype('f8') - original.astype('f8'))
difference[~mask[:, :, axial_middle].T] = 0

fig, ax = plt.subplots(1, 3)
ax[0].imshow(original, cmap='gray', origin='lower')
ax[0].set_title('Original')
ax[1].imshow(final_output, cmap='gray', origin='lower')
ax[1].set_title('ASCM output')
ax[2].imshow(difference, cmap='gray', origin='lower')
ax[2].set_title('Residual')
for i in range(3):
    ax[i].set_axis_off()

plt.savefig('denoised_ascm.png', bbox_inches='tight')

print("The ascm result saved in denoised_ascm.png")

"""
.. figure:: denoised_ascm.png
   :align: center

   Showing the axial slice without (left) and with (middle) ASCM denoising.
"""

"""
From the above figure we can see that the residual is really uniform in nature
which dictates that ASCM denoises the data while preserving the sharpness of
the features.
"""

nib.save(nib.Nifti1Image(den_final, affine), 'denoised_ascm.nii.gz')

print("Saving the entire denoised output in denoised_ascm.nii.gz")

"""
For comparison propose we also plot the outputs of the ``non_local_means``
(both with the larger as well as with the smaller patch radius) with the ASCM
output.
"""

fig, ax = plt.subplots(1, 4)
ax[0].imshow(original, cmap='gray', origin='lower')
ax[0].set_title('Original')
ax[1].imshow(den_small[..., axial_middle].T, cmap='gray', origin='lower',
             interpolation='none')
ax[1].set_title('NLMEANS small')
ax[2].imshow(den_large[..., axial_middle].T, cmap='gray', origin='lower',
             interpolation='none')
ax[2].set_title('NLMEANS large')
ax[3].imshow(final_output, cmap='gray', origin='lower', interpolation='none')
ax[3].set_title('ASCM ')
for i in range(4):
    ax[i].set_axis_off()

plt.savefig('ascm_comparison.png', bbox_inches='tight')

print("The comparison result saved in ascm_comparison.png")

"""
.. figure:: ascm_comparison.png
   :align: center

   Comparing outputs of the NLMEANS and ASCM.
"""

"""
From the above figure, we can observe that the information of two pre-denoised
versions of the raw data, ASCM outperforms standard non-local means in
supressing noise and preserving feature sharpness.

References
----------

..  [Coupe11] Pierrick Coupe, Jose Manjon, Montserrat Robles, Louis Collins.
    Adaptive Multiresolution Non-Local Means Filter for 3D MR Image Denoising.
    IET Image Processing, Institution of Engineering and Technology,
    2011. <00645538>

.. include:: ../links_names.inc

"""



import logging
import numpy as np
from nose.tools import assert_equal
from os.path import join
import numpy.testing as npt

import nibabel as nib
from dipy.io.peaks import load_peaks
from nibabel.tmpdirs import TemporaryDirectory

from dipy.data import get_data
from dipy.workflows.reconst import ReconstCSDFlow, ReconstCSAFlow
logging.getLogger().setLevel(logging.INFO)


def test_reconst_csa():
    reconst_flow_core(ReconstCSAFlow)


def test_reconst_csd():
    reconst_flow_core(ReconstCSDFlow)


def reconst_flow_core(flow):
    with TemporaryDirectory() as out_dir:
        data_path, bval_path, bvec_path = get_data('small_64D')
        vol_img = nib.load(data_path)
        volume = vol_img.get_data()
        mask = np.ones_like(volume[:, :, :, 0])
        mask_img = nib.Nifti1Image(mask.astype(np.uint8), vol_img.affine)
        mask_path = join(out_dir, 'tmp_mask.nii.gz')
        nib.save(mask_img, mask_path)

        reconst_flow = flow()

        reconst_flow.run(data_path, bval_path, bvec_path, mask_path,
                         out_dir=out_dir, extract_pam_values=True)

        gfa_path = reconst_flow.last_generated_outputs['out_gfa']
        gfa_data = nib.load(gfa_path).get_data()
        assert_equal(gfa_data.shape, volume.shape[:-1])

        peaks_dir_path = reconst_flow.last_generated_outputs['out_peaks_dir']
        peaks_dir_data = nib.load(peaks_dir_path).get_data()
        assert_equal(peaks_dir_data.shape[-1], 15)
        assert_equal(peaks_dir_data.shape[:-1], volume.shape[:-1])

        peaks_idx_path = \
            reconst_flow.last_generated_outputs['out_peaks_indices']
        peaks_idx_data = nib.load(peaks_idx_path).get_data()
        assert_equal(peaks_idx_data.shape[-1], 5)
        assert_equal(peaks_idx_data.shape[:-1], volume.shape[:-1])

        peaks_vals_path = \
            reconst_flow.last_generated_outputs['out_peaks_values']
        peaks_vals_data = nib.load(peaks_vals_path).get_data()
        assert_equal(peaks_vals_data.shape[-1], 5)
        assert_equal(peaks_vals_data.shape[:-1], volume.shape[:-1])

        shm_path = reconst_flow.last_generated_outputs['out_shm']
        shm_data = nib.load(shm_path).get_data()
        assert_equal(shm_data.shape[-1], 45)
        assert_equal(shm_data.shape[:-1], volume.shape[:-1])

        pam = load_peaks(reconst_flow.last_generated_outputs['out_pam'])
        npt.assert_allclose(pam.peak_dirs.reshape(peaks_dir_data.shape),
                            peaks_dir_data)
        npt.assert_allclose(pam.peak_values, peaks_vals_data)
        npt.assert_allclose(pam.peak_indices, peaks_idx_data)
        npt.assert_allclose(pam.shm_coeff, shm_data)
        npt.assert_allclose(pam.gfa, gfa_data)

        if flow.get_short_name() == 'csd':

            reconst_flow = flow()
            reconst_flow._force_overwrite = True
            reconst_flow.run(data_path, bval_path, bvec_path, mask_path,
                             out_dir=out_dir, frf=[15, 5, 5])
            reconst_flow = flow()
            reconst_flow._force_overwrite = True
            reconst_flow.run(data_path, bval_path, bvec_path, mask_path,
                             out_dir=out_dir, frf='15, 5, 5')
            reconst_flow = flow()
            reconst_flow._force_overwrite = True
            reconst_flow.run(data_path, bval_path, bvec_path, mask_path,
                             out_dir=out_dir, frf=None)
            reconst_flow2 = flow()
            reconst_flow2._force_overwrite = True
            reconst_flow2.run(data_path, bval_path, bvec_path, mask_path,
                              out_dir=out_dir, frf=None,
                              roi_center=[10, 10, 10])


if __name__ == '__main__':
    test_reconst_csa()
    test_reconst_csd()



from __future__ import division, print_function, absolute_import
import logging
from dipy.align.reslice import reslice
from dipy.io.image import load_nifti, save_nifti
from dipy.workflows.workflow import Workflow


class ResliceFlow(Workflow):
    
    @classmethod
    def get_short_name(cls):
        return 'reslice'

    def run(self, input_files, new_vox_size, order=1, mode='constant', cval=0,
            num_processes=1, out_dir='', out_resliced='resliced.nii.gz'):
        """Reslice data with new voxel resolution defined by ``new_vox_sz``
    
        Parameters
        ----------
        input_files : string
            Path to the input volumes. This path may contain wildcards to
            process multiple inputs at once.
        new_vox_size : variable float
            new voxel size
        order : int, optional
            order of interpolation, from 0 to 5, for resampling/reslicing,
            0 nearest interpolation, 1 trilinear etc.. if you don't want any 
            smoothing 0 is the option you need (default 1)
        mode : string, optional
            Points outside the boundaries of the input are filled according
            to the given mode 'constant', 'nearest', 'reflect' or 'wrap'
            (default 'constant')
        cval : float, optional
            Value used for points outside the boundaries of the input if
            mode='constant' (default 0)
        num_processes : int, optional
            Split the calculation to a pool of children processes. This only
            applies to 4D `data` arrays. If a positive integer then it defines
            the size of the multiprocessing pool that will be used. If 0, then
            the size of the pool will equal the number of cores available.
            (default 1)
        out_dir : string, optional
            Output directory (default input file directory)
        out_resliced : string, optional
            Name of the resliced dataset to be saved
            (default 'resliced.nii.gz')
        """
        
        io_it = self.get_io_iterator()

        for inputfile, outpfile in io_it:
            
            data, affine, vox_sz = load_nifti(inputfile, return_voxsize=True)
            logging.info('Processing {0}'.format(inputfile))
            new_data, new_affine = reslice(data, affine, vox_sz, new_vox_size,
                                           order, mode=mode, cval=cval,
                                           num_processes=num_processes)
            save_nifti(outpfile, new_data, new_affine)
            logging.info('Resliced file save in {0}'.format(outpfile))
        


import os
import sys
import pickle
import numpy as np

from os.path import join as pjoin
import numpy.testing as npt

from dipy.data import read_viz_icons, fetch_viz_icons
from dipy.viz import ui
from dipy.viz import window
from dipy.data import DATA_DIR
from nibabel.tmpdirs import InTemporaryDirectory

from dipy.viz.ui import UI

from dipy.testing.decorators import xvfb_it

# Conditional import machinery for vtk
from dipy.utils.optpkg import optional_package

# Allow import, but disable doctests if we don't have vtk
vtk, have_vtk, setup_module = optional_package('vtk')

use_xvfb = os.environ.get('TEST_WITH_XVFB', False)
if use_xvfb == 'skip':
    skip_it = True
else:
    skip_it = False

if have_vtk:
    print("Using VTK {}".format(vtk.vtkVersion.GetVTKVersion()))


class EventCounter(object):
    def __init__(self, events_names=["CharEvent",
                                     "MouseMoveEvent",
                                     "KeyPressEvent",
                                     "KeyReleaseEvent",
                                     "LeftButtonPressEvent",
                                     "LeftButtonReleaseEvent",
                                     "RightButtonPressEvent",
                                     "RightButtonReleaseEvent",
                                     "MiddleButtonPressEvent",
                                     "MiddleButtonReleaseEvent"]):
        # Events to count
        self.events_counts = {name: 0 for name in events_names}

    def count(self, i_ren, obj, element):
        """ Simple callback that counts events occurences. """
        self.events_counts[i_ren.event.name] += 1

    def monitor(self, ui_component):
        for event in self.events_counts:
            for actor in ui_component.get_actors():
                ui_component.add_callback(actor, event, self.count)

    def save(self, filename):
        with open(filename, 'wb') as f:
            pickle.dump(self.events_counts, f, protocol=2)

    @classmethod
    def load(cls, filename):
        event_counter = cls()
        with open(filename, 'rb') as f:
            event_counter.events_counts = pickle.load(f)

        return event_counter

    def check_counts(self, expected):
        npt.assert_equal(len(self.events_counts),
                         len(expected.events_counts))

        # Useful loop for debugging.
        msg = "{}: {} vs. {} (expected)"
        for event, count in expected.events_counts.items():
            if self.events_counts[event] != count:
                print(msg.format(event, self.events_counts[event], count))

        msg = "Wrong count for '{}'."
        for event, count in expected.events_counts.items():
            npt.assert_equal(self.events_counts[event], count,
                             err_msg=msg.format(event))


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_broken_ui_component():
    class BrokenUI(UI):
        def __init__(self):
            self.actor = vtk.vtkActor()
            super(BrokenUI, self).__init__()

    broken_ui = BrokenUI()
    npt.assert_raises(NotImplementedError, broken_ui.get_actors)
    npt.assert_raises(NotImplementedError, broken_ui.set_center, (1, 2))


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_wrong_interactor_style():
    panel = ui.Panel2D(center=(440, 90), size=(300, 150))
    dummy_renderer = window.Renderer()
    dummy_show_manager = window.ShowManager(dummy_renderer,
                                            interactor_style='trackball')
    npt.assert_raises(TypeError, panel.add_to_renderer, dummy_renderer)


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_rectangle_2d():
    window_size = (700, 700)
    show_manager = window.ShowManager(size=window_size)

    rect = ui.Rectangle2D(size=(100, 50))
    rect.set_position((50, 80))
    npt.assert_equal(rect.position, (50, 80))

    rect.color = (1, 0.5, 0)
    npt.assert_equal(rect.color, (1, 0.5, 0))

    rect.opacity = 0.5
    npt.assert_equal(rect.opacity, 0.5)

    # Check the rectangle is drawn at right place.
    show_manager.ren.add(rect)
    # Uncomment this to start the visualisation
    # show_manager.start()

    colors = [rect.color]
    arr = window.snapshot(show_manager.ren, size=window_size, offscreen=True)
    report = window.analyze_snapshot(arr, colors=colors)
    assert report.objects == 1
    assert report.colors_found

    # Test visibility off.
    rect.set_visibility(False)
    arr = window.snapshot(show_manager.ren, size=window_size, offscreen=True)
    report = window.analyze_snapshot(arr)
    assert report.objects == 0


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_ui_button_panel(recording=False):
    filename = "test_ui_button_panel"
    recording_filename = pjoin(DATA_DIR, filename + ".log.gz")
    expected_events_counts_filename = pjoin(DATA_DIR, filename + ".pkl")

    # Rectangle
    rectangle_test = ui.Rectangle2D(size=(10, 10))
    rectangle_test.get_actors()
    another_rectangle_test = ui.Rectangle2D(size=(1, 1))
    # /Rectangle

    # Button
    fetch_viz_icons()

    icon_files = dict()
    icon_files['stop'] = read_viz_icons(fname='stop2.png')
    icon_files['play'] = read_viz_icons(fname='play3.png')

    button_test = ui.Button2D(icon_fnames=icon_files)
    button_test.set_center((20, 20))

    def make_invisible(i_ren, obj, button):
        # i_ren: CustomInteractorStyle
        # obj: vtkActor picked
        # button: Button2D
        button.set_visibility(False)
        i_ren.force_render()
        i_ren.event.abort()

    def modify_button_callback(i_ren, obj, button):
        # i_ren: CustomInteractorStyle
        # obj: vtkActor picked
        # button: Button2D
        button.next_icon()
        i_ren.force_render()

    button_test.on_right_mouse_button_pressed = make_invisible
    button_test.on_left_mouse_button_pressed = modify_button_callback

    button_test.scale((2, 2))
    button_color = button_test.color
    button_test.color = button_color
    # /Button

    # TextBlock
    text_block_test = ui.TextBlock2D()
    text_block_test.message = 'TextBlock'
    text_block_test.color = (0, 0, 0)

    # Panel
    panel = ui.Panel2D(center=(440, 90), size=(300, 150),
                       color=(1, 1, 1), align="right")
    panel.add_element(rectangle_test, 'absolute', (580, 150))
    panel.add_element(button_test, 'relative', (0.2, 0.2))
    panel.add_element(text_block_test, 'relative', (0.7, 0.7))
    npt.assert_raises(ValueError, panel.add_element, another_rectangle_test,
                      'error_string', (1, 2))
    # /Panel

    # Assign the counter callback to every possible event.
    event_counter = EventCounter()
    event_counter.monitor(button_test)
    event_counter.monitor(panel)

    current_size = (600, 600)
    show_manager = window.ShowManager(size=current_size, title="DIPY Button")

    show_manager.ren.add(panel)

    if recording:
        show_manager.record_events_to_file(recording_filename)
        print(list(event_counter.events_counts.items()))
        event_counter.save(expected_events_counts_filename)

    else:
        show_manager.play_events_from_file(recording_filename)
        expected = EventCounter.load(expected_events_counts_filename)
        event_counter.check_counts(expected)


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_ui_textbox(recording=False):
    filename = "test_ui_textbox"
    recording_filename = pjoin(DATA_DIR, filename + ".log.gz")
    expected_events_counts_filename = pjoin(DATA_DIR, filename + ".pkl")

    # TextBox
    textbox_test = ui.TextBox2D(height=3, width=10, text="Text")

    another_textbox_test = ui.TextBox2D(height=3, width=10, text="Enter Text")
    another_textbox_test.set_message("Enter Text")
    another_textbox_test.set_center((10, 100))
    # /TextBox

    # Assign the counter callback to every possible event.
    event_counter = EventCounter()
    event_counter.monitor(textbox_test)

    current_size = (600, 600)
    show_manager = window.ShowManager(size=current_size, title="DIPY TextBox")

    show_manager.ren.add(textbox_test)

    if recording:
        show_manager.record_events_to_file(recording_filename)
        print(list(event_counter.events_counts.items()))
        event_counter.save(expected_events_counts_filename)

    else:
        show_manager.play_events_from_file(recording_filename)
        expected = EventCounter.load(expected_events_counts_filename)
        event_counter.check_counts(expected)


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_text_block_2d():
    text_block = ui.TextBlock2D()

    def _check_property(obj, attr, values):
        for value in values:
            setattr(obj, attr, value)
            npt.assert_equal(getattr(obj, attr), value)

    _check_property(text_block, "bold", [True, False])
    _check_property(text_block, "italic", [True, False])
    _check_property(text_block, "shadow", [True, False])
    _check_property(text_block, "font_size", range(100))
    _check_property(text_block, "message", ["", "Hello World", "Line\nBreak"])
    _check_property(text_block, "justification", ["left", "center", "right"])
    _check_property(text_block, "position", [(350, 350), (0.5, 0.5)])
    _check_property(text_block, "color", [(0., 0.5, 1.)])
    _check_property(text_block, "background_color", [(0., 0.5, 1.), None])
    _check_property(text_block, "vertical_justification",
                        ["top", "middle", "bottom"])
    _check_property(text_block, "font_family", ["Arial", "Courier"])

    with npt.assert_raises(ValueError):
        text_block.font_family = "Verdana"

    with npt.assert_raises(ValueError):
        text_block.justification = "bottom"

    with npt.assert_raises(ValueError):
        text_block.vertical_justification = "left"


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_text_block_2d_justification():
    window_size = (700, 700)
    show_manager = window.ShowManager(size=window_size)

    # To help visualize the text positions.
    grid_size = (500, 500)
    bottom, middle, top = 50, 300, 550
    left, center, right = 50, 300, 550
    line_color = (1, 0, 0)

    grid_top = (center, top), (grid_size[0], 1)
    grid_bottom = (center, bottom), (grid_size[0], 1)
    grid_left = (left, middle), (1, grid_size[1])
    grid_right = (right, middle), (1, grid_size[1])
    grid_middle = (center, middle), (grid_size[0], 1)
    grid_center = (center, middle), (1, grid_size[1])
    grid_specs = [grid_top, grid_bottom, grid_left, grid_right,
                  grid_middle, grid_center]
    for spec in grid_specs:
        line = ui.Rectangle2D(center=spec[0], size=spec[1], color=line_color)
        show_manager.ren.add(line)

    font_size = 60
    bg_color = (1, 1, 1)
    texts = []
    texts += [ui.TextBlock2D("HH", position=(left, top),
                             font_size=font_size,
                             color=(1, 0, 0), bg_color=bg_color,
                             justification="left",
                             vertical_justification="top")]
    texts += [ui.TextBlock2D("HH", position=(center, top),
                             font_size=font_size,
                             color=(0, 1, 0), bg_color=bg_color,
                             justification="center",
                             vertical_justification="top")]
    texts += [ui.TextBlock2D("HH", position=(right, top),
                             font_size=font_size,
                             color=(0, 0, 1), bg_color=bg_color,
                             justification="right",
                             vertical_justification="top")]

    texts += [ui.TextBlock2D("HH", position=(left, middle),
                             font_size=font_size,
                             color=(1, 1, 0), bg_color=bg_color,
                             justification="left",
                             vertical_justification="middle")]
    texts += [ui.TextBlock2D("HH", position=(center, middle),
                             font_size=font_size,
                             color=(0, 1, 1), bg_color=bg_color,
                             justification="center",
                             vertical_justification="middle")]
    texts += [ui.TextBlock2D("HH", position=(right, middle),
                             font_size=font_size,
                             color=(1, 0, 1), bg_color=bg_color,
                             justification="right",
                             vertical_justification="middle")]

    texts += [ui.TextBlock2D("HH", position=(left, bottom),
                             font_size=font_size,
                             color=(0.5, 0, 1), bg_color=bg_color,
                             justification="left",
                             vertical_justification="bottom")]
    texts += [ui.TextBlock2D("HH", position=(center, bottom),
                             font_size=font_size,
                             color=(1, 0.5, 0), bg_color=bg_color,
                             justification="center",
                             vertical_justification="bottom")]
    texts += [ui.TextBlock2D("HH", position=(right, bottom),
                             font_size=font_size,
                             color=(0, 1, 0.5), bg_color=bg_color,
                             justification="right",
                             vertical_justification="bottom")]

    show_manager.ren.add(*texts)

    # Uncomment this to start the visualisation
    # show_manager.start()

    arr = window.snapshot(show_manager.ren, size=window_size, offscreen=True)
    if vtk.vtkVersion.GetVTKVersion() == "6.0.0":
        expected = np.load(pjoin(DATA_DIR, "test_ui_text_block.npz"))
        npt.assert_array_almost_equal(arr, expected["arr_0"])


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_ui_line_slider_2d(recording=False):
    filename = "test_ui_line_slider_2d"
    recording_filename = pjoin(DATA_DIR, filename + ".log.gz")
    expected_events_counts_filename = pjoin(DATA_DIR, filename + ".pkl")

    line_slider_2d_test = ui.LineSlider2D(initial_value=-2,
                                          min_value=-5, max_value=5)
    line_slider_2d_test.set_center((300, 300))

    # Assign the counter callback to every possible event.
    event_counter = EventCounter()
    event_counter.monitor(line_slider_2d_test)

    current_size = (600, 600)
    show_manager = window.ShowManager(size=current_size,
                                      title="DIPY Line Slider")

    show_manager.ren.add(line_slider_2d_test)

    if recording:
        show_manager.record_events_to_file(recording_filename)
        print(list(event_counter.events_counts.items()))
        event_counter.save(expected_events_counts_filename)

    else:
        show_manager.play_events_from_file(recording_filename)
        expected = EventCounter.load(expected_events_counts_filename)
        event_counter.check_counts(expected)


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_ui_disk_slider_2d(recording=False):
    filename = "test_ui_disk_slider_2d"
    recording_filename = pjoin(DATA_DIR, filename + ".log.gz")
    expected_events_counts_filename = pjoin(DATA_DIR, filename + ".pkl")

    disk_slider_2d_test = ui.DiskSlider2D()
    disk_slider_2d_test.set_center((300, 300))
    disk_slider_2d_test.value = 90

    # Assign the counter callback to every possible event.
    event_counter = EventCounter()
    event_counter.monitor(disk_slider_2d_test)

    current_size = (600, 600)
    show_manager = window.ShowManager(size=current_size,
                                      title="DIPY Disk Slider")

    show_manager.ren.add(disk_slider_2d_test)

    if recording:
        # Record the following events
        # 1. Left Click on the disk and hold it
        # 2. Move to the left the disk and make 1.5 tour
        # 3. Release the disk
        # 4. Left Click on the disk and hold it
        # 5. Move to the right the disk and make 1 tour
        # 6. Release the disk
        show_manager.record_events_to_file(recording_filename)
        print(list(event_counter.events_counts.items()))
        event_counter.save(expected_events_counts_filename)

    else:
        show_manager.play_events_from_file(recording_filename)
        expected = EventCounter.load(expected_events_counts_filename)
        event_counter.check_counts(expected)


@npt.dec.skipif(not have_vtk or skip_it)
@xvfb_it
def test_ui_file_select_menu_2d(recording=False):
    filename = "test_ui_file_select_menu_2d"
    recording_filename = pjoin(DATA_DIR, filename + ".log.gz")
    expected_events_counts_filename = pjoin(DATA_DIR, filename + ".pkl")
    with InTemporaryDirectory():
        for i in range(10):
            _ = open("test" + str(i) + ".txt", 'wt').write('some text')

        file_select_menu = ui.FileSelectMenu2D(size=(500, 500),
                                               position=(300, 300),
                                               font_size=16,
                                               extensions=["txt"],
                                               directory_path=os.getcwd(),
                                               parent=None)
        file_select_menu.set_center((300, 300))

        npt.assert_equal(file_select_menu.text_item_list[1].file_name[:4], "test")
        npt.assert_equal(file_select_menu.text_item_list[5].file_name[:4], "test")

        event_counter = EventCounter()
        for event in event_counter.events_counts:
            file_select_menu.add_callback(file_select_menu.buttons["up"].actor,
                                          event, event_counter.count)
            file_select_menu.add_callback(file_select_menu.buttons["down"].actor,
                                          event, event_counter.count)
            file_select_menu.menu.add_callback(file_select_menu.menu.panel.actor,
                                               event, event_counter.count)
            for text_ui in file_select_menu.text_item_list:
                file_select_menu.add_callback(text_ui.text_actor.get_actors()[0],
                                              event, event_counter.count)

        current_size = (600, 600)
        show_manager = window.ShowManager(size=current_size,
                                          title="DIPY File Select Menu")
        show_manager.ren.add(file_select_menu)

        if recording:
            show_manager.record_events_to_file(recording_filename)
            print(list(event_counter.events_counts.items()))
            event_counter.save(expected_events_counts_filename)

        else:
            show_manager.play_events_from_file(recording_filename)
            expected = EventCounter.load(expected_events_counts_filename)
            event_counter.check_counts(expected)

if __name__ == "__main__":
    if len(sys.argv) <= 1 or sys.argv[1] == "test_ui_button_panel":
        test_ui_button_panel(recording=True)

    if len(sys.argv) <= 1 or sys.argv[1] == "test_ui_textbox":
        test_ui_textbox(recording=True)

    if len(sys.argv) <= 1 or sys.argv[1] == "test_ui_line_slider_2d":
        test_ui_line_slider_2d(recording=True)

    if len(sys.argv) <= 1 or sys.argv[1] == "test_ui_disk_slider_2d":
        test_ui_disk_slider_2d(recording=True)

    if len(sys.argv) <= 1 or sys.argv[1] == "test_ui_file_select_menu_2d":
        test_ui_file_select_menu_2d(recording=True)



"""
=================================================
Particle Filtering Tractography
=================================================
Particle Filtering Tractography (PFT) [Girard2014]_ uses tissue partial
volume estimation (PVE) to reconstruct trajectories connecting the gray matter,
and not incorrectly stopping in the white matter or in the corticospinal fluid.
It relies on a tissue classifier that identifies the tissue where the
streamline stopped. If the streamline correctly stopped in the gray matter, the
trajectory is kept. If the streamline incorrecly stopped in the white matter or
in the corticospinal fluid, PFT uses anatomical information to find an
alternative streamline segment to extend the trajectory. When this segment is
found, the tractography continues until the streamline correctly stops in the
gray matter.

PFT finds an alternative streamline segment whenever the tissue classifier
returns a position classified as 'INVALIDPOINT'.

This example is an extension of the
:ref:`probabilistic_fiber_tracking` example. We begin by loading the
data, fitting a Constrained Spherical Deconvolution (CSD) reconstruction
model and creating the probabilistic direction getter.
"""

import numpy as np

from dipy.data import (read_stanford_labels, default_sphere,
                       read_stanford_pve_maps)
from dipy.direction import ProbabilisticDirectionGetter
from dipy.io.trackvis import save_trk
from dipy.reconst.csdeconv import (ConstrainedSphericalDeconvModel,
                                   auto_response)
from dipy.tracking.local import LocalTracking, ParticleFilteringTracking
from dipy.tracking import utils
from dipy.viz import window, actor
from dipy.viz.colormap import line_colors


renderer = window.Renderer()

img_pve_csf, img_pve_gm, img_pve_wm = read_stanford_pve_maps()
hardi_img, gtab, labels_img = read_stanford_labels()

data = hardi_img.get_data()
labels = labels_img.get_data()
affine = hardi_img.get_affine()
shape = labels.shape

response, ratio = auto_response(gtab, data, roi_radius=10, fa_thr=0.7)
csd_model = ConstrainedSphericalDeconvModel(gtab, response)
csd_fit = csd_model.fit(data, mask=img_pve_wm.get_data())

dg = ProbabilisticDirectionGetter.from_shcoeff(csd_fit.shm_coeff,
                                               max_angle=20.,
                                               sphere=default_sphere)


"""
CMC/ACT Tissue Classifiers
---------------------
Continuous map criterion (CMC) [Girard2014]_ and Anatomically-constrained
tractography (ACT) [Smith2012]_ both uses PVEs information from
anatomical images to determine when the tractography stops.
Both tissue classifiers use a trilinear interpolation
at the tracking position. CMC tissue classifier uses a probability derived from
the PVE maps to determine if the streamline reaches a 'valid' or 'invalid'
region. ACT uses a fixed threshold on the PVE maps. Both tissue classifiers can
be used in conjunction with PFT. In this example, we used CMC.
"""

from dipy.tracking.local import CmcTissueClassifier
from dipy.tracking.streamline import Streamlines

voxel_size = np.average(img_pve_wm.get_header()['pixdim'][1:4])
step_size = 0.2

cmc_classifier = CmcTissueClassifier.from_pve(img_pve_wm.get_data(),
                                              img_pve_gm.get_data(),
                                              img_pve_csf.get_data(),
                                              step_size=step_size,
                                              average_voxel_size=voxel_size)

# seeds are place in voxel of the corpus callosum containing only white matter
seed_mask = labels == 2
seed_mask[img_pve_wm.get_data() < 0.5] = 0
seeds = utils.seeds_from_mask(seed_mask, density=2, affine=affine)

# Particle Filtering Tractography
pft_streamline_generator = ParticleFilteringTracking(dg,
                                                     cmc_classifier,
                                                     seeds,
                                                     affine,
                                                     max_cross=1,
                                                     step_size=step_size,
                                                     maxlen=1000,
                                                     pft_back_tracking_dist=2,
                                                     pft_front_tracking_dist=1,
                                                     particle_count=15,
                                                     return_all=False)

#streamlines = list(pft_streamline_generator)                                                     
streamlines = Streamlines(pft_streamline_generator)
save_trk("pft_streamline.trk", streamlines, affine, shape)


renderer.clear()
renderer.add(actor.line(streamlines, line_colors(streamlines)))
window.record(renderer, out_path='pft_streamlines.png', size=(600, 600))

"""
.. figure:: pft_streamlines.png
 :align: center

 **Particle Filtering Tractography**
"""

# Local Probabilistic Tractography
prob_streamline_generator = LocalTracking(dg,
                                          cmc_classifier,
                                          seeds,
                                          affine,
                                          max_cross=1,
                                          step_size=step_size,
                                          maxlen=1000,
                                          return_all=False)
#streamlines = list(pro)
streamlines = Streamlines(prob_streamline_generator)
save_trk("probabilistic_streamlines.trk", streamlines, affine, shape)

renderer.clear()
renderer.add(actor.line(streamlines, line_colors(streamlines)))
window.record(renderer, out_path='probabilistic_streamlines.png',
              size=(600, 600))

"""
.. figure:: probabilistic_streamlines.png
 :align: center

 **Probabilistic Tractography**
"""

"""
References
----------
.. [Girard2014] Girard, G., Whittingstall, K., Deriche, R., & Descoteaux, M.
    Towards quantitative connectivity analysis: reducing tractography biases.
    NeuroImage, 98, 266-278, 2014.

.. [Smith2012] Smith, R. E., Tournier, J.-D., Calamante, F., & Connelly, A.
    Anatomically-constrained tractography: Improved diffusion MRI
    streamlines tractography through effective use of anatomical
    information. NeuroImage, 63(3), 1924-1938, 2012.
"""



import numpy as np
import nibabel as nib
from numpy.testing import (assert_equal, run_module_suite,
                           assert_array_almost_equal)
from dipy.data import get_data
from dipy.tracking.streamline import Streamlines
from dipy.align.streamlinear import whole_brain_slr, slr_with_qb
from dipy.tracking.distances import bundles_distances_mam
from dipy.align.streamlinear import transform_streamlines
from dipy.align.streamlinear import compose_matrix44, decompose_matrix44


def test_whole_brain_slr():
    streams, hdr = nib.trackvis.read(get_data('fornix'))
    fornix = [s[0] for s in streams]

    f = Streamlines(fornix)
    f1 = f.copy()
    f2 = f.copy()

    # check translation
    f2._data += np.array([50, 0, 0])

    moved, transform, qb_centroids1, qb_centroids2 = whole_brain_slr(
            f1, f2, verbose=True, rm_small_clusters=2, greater_than=0,
            less_than=np.inf, qb_thr=5, progressive=False)

    # we can check the quality of registration by comparing the matrices
    # MAM streamline distances before and after SLR
    D12 = bundles_distances_mam(f1, f2)
    D1M = bundles_distances_mam(f1, moved)

    d12_minsum = np.sum(np.min(D12, axis=0))
    d1m_minsum = np.sum(np.min(D1M, axis=0))

    assert_equal(d1m_minsum < d12_minsum, True)

    assert_array_almost_equal(transform[:3, 3], [-50, -0, -0], 3)

    # check rotation
    mat = compose_matrix44([0, 0, 0, 15, 0, 0])

    f3 = f.copy()
    f3 = transform_streamlines(f3, mat)

    moved, transform, qb_centroids1, qb_centroids2 = slr_with_qb(
            f1, f3, verbose=False, rm_small_clusters=1, greater_than=20,
            less_than=np.inf, qb_thr=2, progressive=True)

    # we can also check the quality by looking at the decomposed transform
    assert_array_almost_equal(decompose_matrix44(transform)[3], -15, 2)

    moved, transform, qb_centroids1, qb_centroids2 = slr_with_qb(
            f1, f3, verbose=False, rm_small_clusters=1, select_random=400,
            greater_than=20,
            less_than=np.inf, qb_thr=2, progressive=True)

    # we can also check the quality by looking at the decomposed transform
    assert_array_almost_equal(decompose_matrix44(transform)[3], -15, 2)


if __name__ == '__main__':
    run_module_suite()



from dipy.data import get_data
from dipy.workflows.io import IoInfoFlow
import logging

import numpy as np
from tempfile import mkstemp
fname_log = mkstemp()[1]

logging.basicConfig(level=logging.INFO,
                    format='%(levelname)s %(message)s',
                    filename=fname_log,
                    filemode='w')


def test_io_info():
    fimg, fbvals, fbvecs=get_data('small_101D')
    io_info_flow = IoInfoFlow()
    io_info_flow.run([fimg, fbvals, fbvecs])
   
    fimg, fbvals, fvecs = get_data('small_25')
    io_info_flow = IoInfoFlow()
    io_info_flow.run([fimg, fbvals, fvecs])
    
    io_info_flow = IoInfoFlow()
    io_info_flow.run([fimg, fbvals, fvecs], b0_threshold=20, bvecs_tol=0.001) 

    file = open(fname_log, 'r')
    lines = file.readlines()
    try:
        np.testing.assert_equal(
            lines[-3],
            'INFO Total number of unit bvectors 25\n')
    except IndexError: # logging maybe disabled in IDE setting
        pass
    file.close()
        
if __name__ == '__main__':
    test_io_info()


import numpy as np
import numpy.testing as npt
from dipy.denoise.noise_estimate import estimate_sigma
from dipy.denoise.nlmeans import nlmeans
import dipy.data as dpd
import nibabel as nib


def test_denoise():
    """

    """
    fdata, fbval, fbvec = dpd.get_data()
    # Test on 4D image:
    data = nib.load(fdata).get_data()
    sigma1 = estimate_sigma(data)
    nlmeans(data, sigma=sigma1)

    # Test on 3D image:
    data = data[..., 0]
    sigma2 = estimate_sigma(data)
    nlmeans(data, sigma=sigma2)



import nibabel as nib
from nose.tools import assert_equal
from dipy.data import get_data
from dipy.segment.quickbundles import QuickBundles


def test_qbundles():
    streams, hdr = nib.trackvis.read(get_data('fornix'))
    T = [s[0] for s in streams]
    qb = QuickBundles(T, 10., 12)
    qb.virtuals()
    qb.exemplars()
    assert_equal(4, qb.total_clusters)
